function VBIContextMenuDataLib(parent) {
    this.parent = parent;
    this.drillUpDimensions = [];
}
VBIContextMenuDataLib.prototype.getLassoItems = function() {
    var self = this;
    var parent = this.parent;
    var componentObject = parent.componentObject;
    var component = componentObject.$();
    var highcharts = isHighChart(component)
    if (!highcharts) {
        return {};
    }
    var items = {};
    items['lasso'] = {
        name: "Lasso",
        icon: highcharts.lasso == "lasso" ? "fa-check" : "",
        callback: function() {
             var component = componentObject.$();
             var highcharts = isHighChart(component)
            highcharts.lasso = "lasso";
            componentObject.lassoState = "lasso";
        }
    }
    items['rlasso'] = {
        name: "Reverse Lasso",
        icon: highcharts.lasso == "rlasso" ? "fa-check" : "",
        callback: function() {
             var component = componentObject.$();
             var highcharts = isHighChart(component)
            highcharts.lasso = "rlasso";
            componentObject.lassoState = "rlasso";
        }
    }
    items['zoom'] = {
        name: "Zoom",
        icon: (highcharts.lasso == "zoom" || (!highcharts.lasso)) ? "fa-check" : "",
        callback: function() {
             var component = componentObject.$();
             var highcharts = isHighChart(component)
            highcharts.lasso = "zoom";
            componentObject.lassoState = "zoom";
        }
    }

    return items;

}
VBIContextMenuDataLib.prototype.drillUpOneLevel = function() {
    var componentObject = this.parent.componentObject;
    var filterItems = componentObject.contextMenuDrillDownFilter.filterItems;
    if (filterItems.length == 0) {
        return
    }
    this.performDrillUp(componentObject, filterItems.length - 1);
}
VBIContextMenuDataLib.prototype.filterInfoPane = function() {
    var self = this;
    var parent = this.parent;
    var componentObject = parent.componentObject;
    var component = componentObject.$();
    var filterItems = componentObject.contextMenuDrillDownFilter.filterItems;
    if (filterItems.length == 0) {
        return
    }
    var tdLength = _.keys(filterItems[0].selectedValuesTextMap).length + 2;
    var layer = $("<div class='vbi-context-filter-panel-layer'></div>");
    var closeLayer = $("<div class='vbi-filter-panel-close-layer'></div>");
    var drillDownTable = $('<table></table>');
    var clearFilterButtonTd = $('<td  colspan="' + tdLength + '">Reset Drilldown</td>');
    var clearFilterTr = $('<tr class="vbi-context-reset-drill-down"></tr>');
    clearFilterTr.append(clearFilterButtonTd);
    drillDownTable.append(clearFilterTr);
    var addItemToDrillDownTable = function(filterItem, index) {
        var oldDimensionFilterTd = "";
        var prepareFilterTd = function(filterTextList, filterDimension) {
            oldDimensionFilterTd = oldDimensionFilterTd + '<td>' + filterItem.dimensionKeyTextPair[filterDimension] + '<ul><li>' + filterTextList.join('</li><li>') + '</li></ul></td>'
        }
        _.each(filterItem.selectedValuesTextMap, prepareFilterTd);
        var itemTr = $('<tr>' + oldDimensionFilterTd + '<td><i class="fa fa-arrow-right" aria-hidden="true"></i></td><td>' + filterItem.newDimension.text + '</td></tr>');
        drillDownTable.append(itemTr);
        if (filterItems.length != (index + 1))
            itemTr.click(function() {
                self.performDrillUp(componentObject, index + 1);
            });
    }
    _.each(filterItems, addItemToDrillDownTable);
    layer.append(drillDownTable);
    component.append(layer);
    $('body').append(closeLayer);
    clearFilterTr.click(function() {
        self.performDrillUp(componentObject, 0);
    });
    closeLayer.click(function() {
        layer.remove();
        closeLayer.remove();
    })
}
VBIContextMenuDataLib.prototype.getDrillUpItems = function() {
    var self = this;
    var parent = this.parent;
    var componentObject = parent.componentObject;
    if (componentObject.contextMenuDrillDownFilter.filterItems.length == 0) {
        return {};
    }
    var filterItems = componentObject.contextMenuDrillDownFilter.filterItems;
    var items = {};
    var addDrillUpItems = function(filterItem, index) {
        var dimensionName = filterItem.oldDimension.text;
        if (index == 0) {
            dimensionName = "Reset DrillDown";
        }
        items[filterItem.oldDimension.key] = {
            name: dimensionName,
            callback: function(itemIndex) {
                self.performDrillUp(componentObject, itemIndex);
            }.bind(null, index)
        }
    }
    _.each(filterItems, addDrillUpItems);
    return items;

}
VBIContextMenuDataLib.prototype.performDrillUp = function(componentObject, itemIndex) {
    var filterItems = componentObject.contextMenuDrillDownFilter.filterItems;
    var itemsToRemoveFilter = _.rest(filterItems, itemIndex);
    var swapDimension = filterItems[itemIndex].oldDimension.key;
    var filterStringMap = {};

    componentObject.callZTLFunction('swapTwoDimensions', function() {}, filterItems[filterItems.length - 1].newDimension.key, swapDimension);

    var existingFilters = _.last(componentObject.contextMenuDrillDownFilter.filterItems).filterStringMap;
    var clearDimensionsFilter = function(filterItem) {
        if (componentObject.existingDsFilters[filterItem.oldDimension.key]) {
            componentObject.callZTLFunction('setFilterExt', function() {}, filterItem.oldDimension.key, componentObject.existingDsFilters[filterItem.oldDimension.key]);
        } else {
            componentObject.callZTLFunction('clearFilter', function() {}, filterItem.oldDimension.key);
        }
        delete componentObject.contextMenuDrillDownFilter.filterdDimesions[filterItem.oldDimension.key];
    }
    var removeFilter = function(dimension) {
        if (dimension == swapDimension) {
            return;
        }
        if (componentObject.existingDsFilters[dimension]) {
            componentObject.callZTLFunction('setFilterExt', function() {}, dimension, componentObject.existingDsFilters[dimension]);
        } else {
            componentObject.callZTLFunction('clearFilter', function() {}, dimension);

        }
    }
    var filertItems = function(filterString, dimension) {
        if (existingFilters[dimension] != filterString && !(dimension in componentObject.contextMenuDrillDownFilter.filterdDimesions))
            componentObject.callZTLFunction('setFilter', function() {}, dimension, filterString);
    }
    _.each(itemsToRemoveFilter, clearDimensionsFilter);
    if (itemIndex != 0) {
        _.each(filterItems[itemIndex - 1].filterStringMap, filertItems);
    } else {
        _.each(_.keys(filterItems[itemIndex].filterStringMap), removeFilter);
    }

    componentObject.contextMenuDrillDownFilter.filterItems = _.initial(filterItems, (filterItems.length - itemIndex));
}
VBIContextMenuDataLib.prototype.getDrillDownItems = function() {
    var self = this;
    var parent = this.parent;
    var resultProimise = $.Deferred();
    var componentObject = parent.componentObject;
    var selectedValues;
    var dsType = getComponentDataSourceType(componentObject);
    if (dsType != "NATIVE") {
        return {};
    }
    var isValidSelection = self.verifySelectedValues(componentObject);
    if (!isValidSelection) {
        return {};
    }
    selectedValues = componentObject.selectedvalues();
    selectedValues = JSON.parse("[" + selectedValues + "]");
    var dimensions = componentObject.fulljson().dimensions;
    var rowDimensionKeys = self.getRowDimension(dimensions);
    var selectedValuesMap = self.getSelectedDimensionValues(selectedValues, rowDimensionKeys)
    var selectedValuesTextMap = self.getSelectedDimensionText(selectedValues, rowDimensionKeys)

    componentObject.contextMenuSwapGet = 0;
    var processDimensions = function(filterString) {
        componentObject.contextMenuSwapGet = 1;
        var items = {};
        var selectedDimensions = _.keys(selectedValuesMap);
        if (componentObject.contextMenuDrillDownFilter.filterItems.length === 0) {
            componentObject.existingDsFilters = JSON.parse(filterString);
        }
        var allDimensions = componentObject.existingDsFilters.ALL_DIMENSIONS;
        var dimensionKeyTextPair = self.getDimensionKeyTextPair(allDimensions);
        var dimensionKeys = self.getDimensionKeys(dimensions);
        var filterdDimesions = _.keys(componentObject.contextMenuDrillDownFilter.filterdDimesions);
        allDimensions = self.filterDimensions(allDimensions, dimensionKeys, "name", true);
        allDimensions = self.filterDimensions(allDimensions, filterdDimesions, "name", true);
        var filterStringMap = {};
        var addToFilterStringMap = function(memberList, dimension) {
            filterStringMap[dimension] = self.getFilterString(memberList);
        }
        _.each(selectedValuesMap, addToFilterStringMap);
        dimensions = self.removeMeasure(dimensions);
        var addDimensionToItems = function(dimension) {
            items[dimension.name] = {
                name: dimension.text,
                callback: function(selectedDimensions, filterStringMap, swapWithDimension, selectedValuesTextMap) {
                    var newBardata=self.parent.dataUtilityLib.pushMissingDimensions();
                    if(newBardata){
                        componentObject.bardatacustomization(newBardata);
                        componentObject.firePropertiesChanged(["bardatacustomization"]);
                    }
                    var selectedDimension = _.last(selectedDimensions);
                    var existingFilters = _.last(componentObject.contextMenuDrillDownFilter.filterItems);
                    if (existingFilters) {
                        existingFilters = existingFilters.filterStringMap;
                    } else {
                        existingFilters = {};
                    }
                    var filertItems = function(filterString, dimension) {
                        if (existingFilters[dimension] != filterString) {
                            componentObject.callZTLFunction('setFilter', function() {}, dimension, filterString);
                        }
                    }
                    _.each(filterStringMap, filertItems);
                    componentObject.callZTLFunction('swapTwoDimensions', function() {}, selectedDimension, swapWithDimension);
                    componentObject.contextMenuDrillDownFilter.filterItems.push({
                        "oldDimension": {
                            "key": selectedDimension,
                            "text": dimensionKeyTextPair[selectedDimension]
                        },
                        "selectedDimensions": selectedDimensions,
                        "newDimension": {
                            "key": swapWithDimension,
                            "text": dimensionKeyTextPair[swapWithDimension]
                        },
                        "dimensionKeyTextPair": dimensionKeyTextPair,
                        "selectedValuesTextMap": selectedValuesTextMap,
                        "filterStringMap": filterStringMap
                    })
                    componentObject.contextMenuDrillDownFilter.filterdDimesions[selectedDimension] = "true";
                }.bind(null, selectedDimensions, filterStringMap, dimension.name, selectedValuesTextMap)
            };
        }
        _.each(allDimensions, addDimensionToItems);
        resultProimise.resolve(items);
    }
    self.ztlCallRegister.addItem('getExistingFilters', processDimensions)
    return resultProimise.promise();
}
VBIContextMenuDataLib.prototype.getRowDimension = function(dimensions) {
    var rowDimensionKeys = [];
    var pushToRowDimension = function(dimension) {
        if ((!dimension.containsMeasures) && dimension.axis == "ROWS") {
            rowDimensionKeys.push(dimension.key);
        }
    }
    _.each(dimensions, pushToRowDimension);
    return rowDimensionKeys;
}
VBIContextMenuDataLib.prototype.isMeasureInRow = function(dimensions) {
    var isMeasureInRow = false;
    var verifyDimesions = function(dimension) {
        if (dimension.containsMeasures && dimension.axis == "ROWS") {
            isMeasureInRow = true;
        }
    }
    _.each(dimensions, verifyDimesions);
    return isMeasureInRow;
}
VBIContextMenuDataLib.prototype.getFilterString = function(memberList) {
    var filterString = "";
    var formFilterString = function(member, index) {
        filterString = filterString + member;
        if ((index + 1) != memberList.length) {
            filterString = filterString + "; ";
        }
    }
    _.each(memberList, formFilterString);
    return filterString;
}
VBIContextMenuDataLib.prototype.getConditionalFormatingItmes = function() {
    var R = this.parent.R;
    var componentObject = this.parent.componentObject;
    var originalRuleTable;
    var items = {};
    var currentRuleTable;
    if (!componentObject.originalRuleTable) {
        return {};
    }
    try {
        currentRuleTable = JSON.parse(componentObject.ruletable());
        originalRuleTable = JSON.parse(componentObject.originalRuleTable);
    } catch (e) {
        return {};
    }
    var allRuleKeys = _.keys(originalRuleTable);
    var pushRulesToItems = function(ruleKey) {
        items[ruleKey] = {
            name: " <div style='display:inline-block'><div class='context-contional-formating' style='display:inline-block;background-color:" + originalRuleTable[ruleKey].inputcondformat + "'></div>&nbsp;" + ruleKey + "</div>",
            isHtmlName: true,
            icon: ruleKey in currentRuleTable ? "fa-check" : "",
            callback: function() {
                var selectedKey = arguments[0];
                if (selectedKey in currentRuleTable) {
                    delete currentRuleTable[selectedKey];
                } else {
                    currentRuleTable[selectedKey] = originalRuleTable[selectedKey];
                }
                var ruleKeys = _.keys(currentRuleTable);
                var filertItems = function(rule) {
                    return ruleKeys.indexOf(rule["cfui-ruleName"]) != -1;
                }
                currentRuleTable = R.filter(filertItems, originalRuleTable);
                componentObject.ruletable(currentRuleTable);
                componentObject.firePropertiesChanged(["ruletable"]);
            }

        }
    }
    _.each(allRuleKeys, pushRulesToItems);
    return items;

}
VBIContextMenuDataLib.prototype.getRankingItems = function() {
    var self = this;
    var topList = [5, 10, "N"];
    var bottomList = [5, 10, "N"]
    var items = {}
    var R = self.parent.R;
    var componentObject = this.parent.componentObject;
    var dsType = getComponentDataSourceType(componentObject);
    if (dsType != "NATIVE") {
        return {};
    }
    if (!componentObject.result().dimensions) {
        return {};
    }
    var component = componentObject.$();
    var dimensions = componentObject.fulljson().dimensions;
    var measures = this.getMeasureMebers(dimensions);
    var isMeasureInRow = this.isMeasureInRow(dimensions);
    var rowDimensionKeys = this.getRowDimension(dimensions)
    if (isMeasureInRow || rowDimensionKeys.length >= 3) {
        return {};
    }
    var dataUtilityOptions = componentObject.bardatacustomization();
    var datautility=componentObject.datautility();
    if (!dataUtilityOptions || !datautility) {
        dataUtilityOptions = self.parent.dataUtilityLib.createBarDataCustomization();
    } else {
        try {
            dataUtilityOptions = JSON.parse(dataUtilityOptions);
        } catch (e) {
            return {};
        }
    }
    var layer = $("<div class='vbi-context-rank-layer'></div>");
    var closeLayer = $("<div class='vbi-filter-panel-close-layer'></div>");
    var rankCountField = $("<input class='vbi-rank-input'type='number' placeholder='Enter N Value'></input></br>");
    var okButton = $("<input class='vbi-ok-button' type='button' value='OK'/>");
    var cancelButton = $("<input class='vbi-cancel-button' type='button' value='Cancel'/>");
    var errorMessage = $("<div class='vbi-rank-errmsg'>Invalid Number</div>");
    layer.append(rankCountField).append(okButton).append(cancelButton);

    closeLayer.click(function() {
        layer.remove();
        closeLayer.remove();
    })

    cancelButton.on("click", function() {
        layer.remove();
        closeLayer.remove();
    })
    rankCountField.on("keyup", function() {
        errorMessage.remove();
    })
    rankCountField.on("change", function() {
        errorMessage.remove();
    })

    okButton.on("click", function() {
        var count = parseInt(rankCountField.val())
        if (_.isNaN(count) || count < 0) {
            if (errorMessage.parent().length == 0) {
                layer.append(errorMessage);
            }
            return;
        }

        dataUtilityOptions.topN = count;
        componentObject.datautility(true);
        componentObject.bardatacustomization(JSON.stringify(dataUtilityOptions));
        var measure = rankCountField.attr("measure");
        var isAscending = false;
        if (rankCountField.attr("isAscending") == "true") {
            isAscending = true
        }
        layer.remove();
        closeLayer.remove();
        componentObject.firePropertiesChanged(["bardatacustomization", "datautility"]);
        componentObject.contextRanking = 0;
        componentObject.callZTLFunction("contextMenuSortByMeasure", function() {
            if (componentObject.contextRanking == 1) {
                componentObject.contextRanking = 2
            }
        }, measure, isAscending);

    })

    var addItemsToMenu = function(isAscending, count) {

        var measureItems = {}
        var addMeasureItems = function(measure) {
            measureItems[measure.key] = {
                name: measure.text,
                callback: function(count, measureKey, isAscending) {
                    if (count == "N") {
                        rankCountField.attr("measure", measureKey);
                        rankCountField.attr("isAscending", isAscending);
                        component.append(layer)
                        $('body').append(closeLayer);
                        rankCountField.focus();
                        return;
                    }
                    dataUtilityOptions.topN = count;
                    componentObject.bardatacustomization(JSON.stringify(dataUtilityOptions));
                    componentObject.datautility(true);
                    componentObject.firePropertiesChanged(["bardatacustomization", "datautility"]);
                    componentObject.contextRanking = -1;

                    componentObject.callZTLFunction("contextMenuSortByMeasure", function() {
                        // if (componentObject.contextRanking == 1) {
                        //     componentObject.contextRanking = 2
                        // }
                    }, measureKey, isAscending);


                }.bind(null, count, measure.key, isAscending)
            }
        }
        _.each(measures, addMeasureItems);
        var prefix = "Top"
        if (isAscending)
            prefix = "Bottom"
        items[prefix + count] = {
            name: prefix + " " + count,
            items: measureItems
        }

    }
    var addTopItemsToMenu = R.curry(addItemsToMenu)(false)
    var addBottomItemsToMenu = R.curry(addItemsToMenu)(true)
    _.each(topList, addTopItemsToMenu);
    _.each(bottomList, addBottomItemsToMenu);
    if (dataUtilityOptions.topN) {
        items["clearRank"] = {
            name: "Clear Ranking",
            callback: function() {
                delete dataUtilityOptions.topN;
                componentObject.bardatacustomization(JSON.stringify(dataUtilityOptions));
                componentObject.firePropertiesChanged(["bardatacustomization"]);

            }
        }
    }

    return items;
}
VBIContextMenuDataLib.prototype.getSelectedDimensionValues = function(selectedValues, rowDimensionKeys) {
    var selectedValuesMap = {};
    var pushSelectedValue = function(selectedValue) {
        var pushSelectedMember = function(member, dimension) {
            if (rowDimensionKeys.indexOf(dimension) != -1) {
                if (!selectedValuesMap[dimension]) {
                    selectedValuesMap[dimension] = [];
                }
                if (selectedValuesMap[dimension].indexOf(member.internalKey) === -1)
                    selectedValuesMap[dimension].push(member.internalKey);
            }
        }
        _.each(selectedValue, pushSelectedMember);
    }
    _.each(selectedValues, pushSelectedValue);
    return selectedValuesMap;
}
VBIContextMenuDataLib.prototype.getSelectedDimensionText = function(selectedValues, rowDimensionKeys) {
    var selectedValuesMap = {};
    var pushSelectedValue = function(selectedValue) {
        var pushSelectedMember = function(member, dimension) {
            if (rowDimensionKeys.indexOf(dimension) != -1) {
                if (!selectedValuesMap[dimension]) {
                    selectedValuesMap[dimension] = [];
                }
                if (selectedValuesMap[dimension].indexOf(member.text) === -1)
                    selectedValuesMap[dimension].push(member.text);
            }
        }
        _.each(selectedValue, pushSelectedMember);
    }
    _.each(selectedValues, pushSelectedValue);
    return selectedValuesMap;
}
VBIContextMenuDataLib.prototype.getIndexesForFilter = function(selectedValues, rowDimensionKeys, dimensions) {
    var self = this;
    var memerIndexes = [];
    var dimensionIndexes = [];
    var dimensionIndexMap = {};
    var selectedDimensionKeys = _.keys(selectedValues[0]);
    var selectedRowDimensionKeys = _.intersection(selectedDimensionKeys, rowDimensionKeys);
    var pushToDimensionIndexes = function(dimensionKey) {
        var dimensionIndex = self.getDimensionIndex(dimensions, dimensionKey);
        dimensionIndexes.push(dimensionIndex);
        dimensionIndexMap[dimensionKey] = dimensionIndex;
    }
    _.each(selectedRowDimensionKeys, pushToDimensionIndexes);
    var pushSelectedValue = function(selectedValue) {
        var mebmerIndex = [];
        var pushMemberIndex = function(dimensionKey) {
            mebmerIndex.push(self.getMemberIndex(dimensions, dimensionIndexMap[dimensionKey], [selectedValue[dimensionKey].internalKey])[0]);
        }
        _.each(selectedRowDimensionKeys, pushMemberIndex);
        memerIndexes.push(mebmerIndex);
    }
    _.each(selectedValues, pushSelectedValue);
    return {
        "dimensionIndexes": dimensionIndexes,
        "memerIndexes": memerIndexes
    };
}
VBIContextMenuDataLib.prototype.getMeasureMebers = function(dimensions) {
    var measures = [];
    var loadMeasures = function(dimension) {
        if (dimension.containsMeasures)
            measures = measures.concat(dimension.members)
    }
    _.each(dimensions, loadMeasures);
    return measures;
}
VBIContextMenuDataLib.prototype.getDimensionKeys = function(dimensions) {
    var dimensionKeys = [];
    var loadDimensionKeys = function(dimension) {
        if (!dimension.containsMeasures)
            dimensionKeys.push(dimension.key);
    }
    _.each(dimensions, loadDimensionKeys);
    return dimensionKeys;
}
VBIContextMenuDataLib.prototype.getDimensionKeyTextPair = function(dimensions) {
    var dimensionKeyTextPair = {};
    var loadDimensionKeyTextPair = function(dimension) {
        dimensionKeyTextPair[dimension.name] = dimension.text;
    }
    _.each(dimensions, loadDimensionKeyTextPair);
    return dimensionKeyTextPair;
}
VBIContextMenuDataLib.prototype.filterDimensions = function(dimensions, dimensionKeys, dimensionAttribute, isExclude) {
    if (!dimensionAttribute)
        dimensionAttribute = "key"
    var filterDimension = function(dimension) {
        if (dimension.containsMeasures || dimension.isMeasuresDimension)
            return false;
        if (isExclude) {
            return dimensionKeys.indexOf(dimension[dimensionAttribute]) === -1
        } else {
            return dimensionKeys.indexOf(dimension[dimensionAttribute]) > -1;
        }
    }
    return _.filter(dimensions, filterDimension);
}
VBIContextMenuDataLib.prototype.removeMeasure = function(dimensions) {
    var filterDimension = function(dimension) {
        return !dimension.containsMeasures && !dimension.isMeasuresDimension;

    }
    return _.filter(dimensions, filterDimension);
}
VBIContextMenuDataLib.prototype.getMeasureKey = function(dimensions) {
    var isMeasure = function(dimension) {
        return dimension.containsMeasures;
    }
    dimensions = _.filter(dimensions, isMeasure);
    return dimensions[0].key;
}
VBIContextMenuDataLib.prototype.getSwapWithDimensions = function() {
    var self = this;
    var parent = this.parent;
    var resultProimise = $.Deferred();
    var componentObject = parent.componentObject;
    var dsType = getComponentDataSourceType(componentObject);
    if (dsType != "NATIVE") {
        return {};
    }
    if (!componentObject.result().dimensions) {
        return {};
    }
    var dimensions = componentObject.fulljson().dimensions;
    var processDimensions = function(dimensionsString, returnResult) {
        var items = {};
        componentObject.dimensionString = dimensionsString;
        var allDimensions = JSON.parse(dimensionsString).ALL_DIMENSIONS;
        var dimensionKeys = self.getDimensionKeys(dimensions);
        allDimensions = self.filterDimensions(allDimensions, dimensionKeys, "name", true);
        dimensions = self.removeMeasure(dimensions);
        var addDimensionToItems = function(dimension) {
            var innerItems = {};
            var addDimensionToInnerMenu = function(innerDimension) {
                innerItems[innerDimension.name] = {
                    name: innerDimension.text,
                    callback: function() {
                    var newBardata=self.parent.dataUtilityLib.pushMissingDimensions();
                    if(newBardata){
                        componentObject.bardatacustomization(newBardata);
                        componentObject.firePropertiesChanged(["bardatacustomization"]);
                    }
                        parent.componentObject.callZTLFunction('swapTwoDimensions', function() {}, arguments[1], arguments[0]);
                    }.bind(null, dimension.key)
                };
            }
            _.each(allDimensions, addDimensionToInnerMenu);
            items['outer' + dimension.key] = {
                name: dimension.text,
                items: innerItems
            };
        }
        _.each(dimensions, addDimensionToItems);
        if (returnResult) {
            return items;
        }
        resultProimise.resolve(items);
    }
    if (componentObject.dimensionString) {
        if (self.validateDimesionString(componentObject.dimensionString, dimensions)) {
            return processDimensions(componentObject.dimensionString, true);
        }
    }
    self.ztlCallRegister.addItem('getExistingFilters', processDimensions)
    return resultProimise.promise();
}
VBIContextMenuDataLib.prototype.validateDimesionString = function(dimensionString, dimensions) {
    var allDimensions;
    try {
        allDimensions = JSON.parse(dimensionString).ALL_DIMENSIONS;
    } catch (e) {
        return false;
    }
    var dimensionKeys = this.getDimensionKeys(dimensions);
    var isValid = false;
    var isContainDimensionKeys = function(dimension) {
        if (dimensionKeys.indexOf(dimension.name) != -1) {
            isValid = true;
        }
    }
    _.each(allDimensions, isContainDimensionKeys);
    return isValid;
}
VBIContextMenuDataLib.prototype.getMeasureFilterItems = function() {
    var items = {}
    var self = this;
    var R = self.parent.R;
    var isMeasureSelectionTypeIndex = false;
    var componentObject = this.parent.componentObject;
    if (!componentObject.result().dimensions) {
        return {};
    }
    var dataUtilityOptions = componentObject.bardatacustomization();
    var datautility=componentObject.datautility();
    if (!dataUtilityOptions || !datautility) {
        dataUtilityOptions = self.parent.dataUtilityLib.createBarDataCustomization();
    } else {
        try {
            dataUtilityOptions = JSON.parse(dataUtilityOptions);
        } catch (e) {
            return {};
        }
    }
    if (dataUtilityOptions.MEASURES) {
        if (dataUtilityOptions.MEASURES.length == 0) {
            isMeasureSelectionTypeIndex = true;
        } else if (R.type(dataUtilityOptions.MEASURES[0]) == "Number") {
            isMeasureSelectionTypeIndex = true;
        }
    }
    if (isMeasureSelectionTypeIndex) {
        var measuresList = self.parent.dataUtilityLib.createBarDataCustomization().MEASURES;
        var loadMeasures = function(measure, index) {
            var measureIndex = index + 1;
            var isHidden = dataUtilityOptions.MEASURES.indexOf(measureIndex) == -1;
            items[measure.key] = {
                name: measure.text,
                icon: isHidden ? "" : "fa-check",
                callback: function(measureIndex, isHidden) {
                    if (isHidden) {
                        dataUtilityOptions.MEASURES.push(measureIndex);
                    } else {
                        dataUtilityOptions.MEASURES = _.without(dataUtilityOptions.MEASURES, measureIndex);
                    }
                    componentObject.bardatacustomization(JSON.stringify(dataUtilityOptions));
                    componentObject.datautility(true)
                    componentObject.contextMenuChange = true;
                    componentObject.firePropertiesChanged(['bardatacustomization', 'datautility']);
                }.bind(null, measureIndex, isHidden)
            }
        }
        _.each(measuresList, loadMeasures);
    } else {
        var loadMeasures = function(measure, index) {
            items[measure.key] = {
                name: measure.text,
                icon: measure.show ? "fa-check" : "",
                callback: function() {
                    dataUtilityOptions.MEASURES[arguments[0]].show = arguments[1] ? false : true;
                    componentObject.bardatacustomization(JSON.stringify(dataUtilityOptions));
                    componentObject.datautility(true)
                    componentObject.contextMenuChange = true;
                    componentObject.firePropertiesChanged(['bardatacustomization', 'datautility']);
                }.bind(null, index, measure.show)
            }
        }
        _.each(dataUtilityOptions.MEASURES, loadMeasures);
    }

    return items;
}
VBIContextMenuDataLib.prototype.getMeasureFilterItemsPieFamily = function() {
    var items = {}
    var self = this;
    var R = self.parent.R;
    var measures=self.parent.dataUtilityLib.createBarDataCustomization().MEASURES;
    var isMeasureSelectionTypeIndex = false;
    var componentObject = this.parent.componentObject;
    var selectedMeasure;
    if(componentObject.selectedmeasures){
        selectedMeasure=componentObject.selectedmeasures();
    } 
    else{
        return {};
    }
    if(!selectedMeasure){
        selectedMeasure=measures[0].key;
    }
        var loadMeasures = function(measure, index) {
            items[measure.key] = {
                name: measure.text,
                icon: measure.key== selectedMeasure? "fa-check" : "",
                callback: function(measureKey) {
                    componentObject.selectedmeasures(measureKey);
                    componentObject.contextMenuChange = true;
                    componentObject.firePropertiesChanged(['selectedmeasures']);
                }.bind(null, measure.key)
            }
        }
        _.each(measures, loadMeasures);
    return items;
}
VBIContextMenuDataLib.prototype.getDimensionFilterItems = function() {
    var items = {};
    var self = this;
    var componentObject = self.parent.componentObject;
    if (!componentObject.result().dimensions) {
        return {};
    }
    var dataUtilityOptions = componentObject.bardatacustomization();
    var datautility=componentObject.datautility();
    if (!dataUtilityOptions || !datautility) {    
          dataUtilityOptions = self.parent.dataUtilityLib.createBarDataCustomization();
    } else {
        try {
            dataUtilityOptions = JSON.parse(dataUtilityOptions);
        } catch (e) {
            return {};
        }
    }
    var selectedValues;
    var isValidSelection = self.verifySelectedValues(componentObject);
    if (isValidSelection) {
        selectedValues = componentObject.selectedvalues();
        var dimensions = self.parent.componentObject.fulljson().dimensions;
        var rowDimensionKeys = self.getRowDimension(dimensions);
        selectedValues = JSON.parse("[" + selectedValues + "]");
        var filterTupleObject = self.getIndexesForFilter(selectedValues, rowDimensionKeys, dimensions);
        if (dataUtilityOptions.dimensionIndexes && dataUtilityOptions.memberIndexes) {
            if (JSON.stringify(filterTupleObject.dimensionIndexes) != JSON.stringify(dataUtilityOptions.dimensionIndexes[0])) {
                dataUtilityOptions.dimensionIndexes = [];
                dataUtilityOptions.memberIndexes = [];
                dataUtilityOptions.isMemberExclude = [];
            } else {
                selectedDimensionIndexList = dataUtilityOptions.dimensionIndexes.slice();
                selectedMembereExcludeIndexList = dataUtilityOptions.memberIndexes.slice();
            }

        } else {
            dataUtilityOptions.dimensionIndexes = [];
            dataUtilityOptions.memberIndexes = [];
            dataUtilityOptions.isMemberExclude = [];
        }

        items = {
            "exclude": {
                name: "Exclude",
                callback: function() {
                    dataUtilityOptions.dimensionIndexes.push(filterTupleObject.dimensionIndexes);
                    dataUtilityOptions.memberIndexes.push(filterTupleObject.memerIndexes);
                    dataUtilityOptions.isMemberExclude.push(true);
                    componentObject.bardatacustomization(JSON.stringify(dataUtilityOptions));
                    componentObject.datautility(true);
                    componentObject.selectedvalues('');
                    componentObject.contextMenuChange = true;
                    componentObject.firePropertiesChanged(['bardatacustomization', 'datautility', 'selectedvalues']);

                }
            },
            "include": {
                name: "Include",
                callback: function() {
                    dataUtilityOptions.dimensionIndexes = [filterTupleObject.dimensionIndexes];
                    dataUtilityOptions.memberIndexes = [filterTupleObject.memerIndexes];
                    dataUtilityOptions.isMemberExclude = [false];
                    componentObject.bardatacustomization(JSON.stringify(dataUtilityOptions));
                    componentObject.datautility(true);
                    componentObject.selectedvalues('');
                    componentObject.contextMenuChange = true;
                    componentObject.firePropertiesChanged(['bardatacustomization', 'datautility', 'selectedvalues']);
                }
            }

        }
    }
    if (dataUtilityOptions.memberIndexes || dataUtilityOptions.dimensionIndexes) {
        items["clearFilter"] = {
            name: "Clear Filter",
            callback: function() {
                if (dataUtilityOptions.dimensionIndexes) {
                    delete dataUtilityOptions.dimensionIndexes;
                }
                if (dataUtilityOptions.memberIndexes) {
                    delete dataUtilityOptions.memberIndexes;
                }
                componentObject.bardatacustomization(JSON.stringify(dataUtilityOptions));
                componentObject.datautility(true);
                componentObject.selectedvalues('');
                componentObject.contextMenuChange = true;
                componentObject.firePropertiesChanged(['bardatacustomization', 'datautility', 'selectedvalues']);
            }
        }
    }
    return items;
}
VBIContextMenuDataLib.prototype.getDimensionIndex = function(dimensions, dimensionkey) {
    var dimensionIndex = -1;
    var findIndex = function(object, index) {
        if (object.key == dimensionkey) {
            dimensionIndex = index;
        }
    }
    _.each(dimensions, findIndex);
    return dimensionIndex;
}
VBIContextMenuDataLib.prototype.getMemberIndex = function(dimension, dimensionIndex, membersList, isExclude) {
    var indexes = [];
    var allMember = dimension[dimensionIndex].members;
    var adjustIndex=0;
    if(allMember[0].key=="SUMME")
    {
        adjustIndex=-1; 
    }
    var pushMemeberIndex = function(object, index) {
        if ((membersList.indexOf(object.key) > -1 && !isExclude) || (membersList.indexOf(object.key) == -1 && isExclude)) {
            indexes.push(index+adjustIndex);
        }

    }
    _.each(allMember, pushMemeberIndex)
    return indexes;
}
VBIContextMenuDataLib.prototype.getSortItems = function() {
    var items = {};
    var R = this.parent.R;
    var componentObject = this.parent.componentObject;
    var dsType = getComponentDataSourceType(componentObject);
    if (dsType != "NATIVE") {
        return {};
    }
    var dimensions = this.parent.componentObject.fulljson().dimensions;
    var measures = this.getMeasureMebers(dimensions);
    var addMemberToList = function(option, member) {
        items[member.key] = {
            name: member.text,
            items: {
                ascending: {
                    name: "Ascending",
                    icon: option.ascendingIcon,
                    callback: function() {
                        componentObject.callZTLFunction(option.ztlFunction, function(value) {}, arguments[0], true);
                    }.bind(null, member.key)
                },
                descending: {
                    name: "Descending",
                    icon: option.descendingIcon,
                    callback: function() {
                        componentObject.callZTLFunction(option.ztlFunction, function(value) {}, arguments[0], false);
                    }.bind(null, member.key)
                }
            }
        }
    }
    var isNotMeasure = function(object) {
        return !object.containsMeasures;
    }
    dimensions = _.filter(dimensions, isNotMeasure);
    var addDimensionsToList = R.curry(addMemberToList)({
        'ascendingIcon': 'fa-sort-alpha-asc',
        'descendingIcon': 'fa-sort-alpha-desc',
        'ztlFunction': 'contextMenuSortByDimension'
    })
    var addMeasuresToList = R.curry(addMemberToList)({
        'ascendingIcon': 'fa-sort-numeric-asc',
        'descendingIcon': 'fa-sort-numeric-desc',
        'ztlFunction': 'contextMenuSortByMeasure'
    })
    _.each(measures, addMeasuresToList);
    items.sep1 = "---------"
    _.each(dimensions, addDimensionsToList);
    if (_.keys(items).length == 1)
        return {};
    return items;
}
VBIContextMenuDataLib.prototype.getComponentPropertiesItems = function(properties) {
    var componentObject = this.parent.componentObject;
    var items = {};
    for (var j = 0; j < properties.length; j++) {
        property = properties[j];
        if (componentObject[property.key]) {
            items[property.key] = {
                name: property.text,
                icon: componentObject[property.key]() ? "fa-check" : "",
                callback: function() {
                    componentObject[arguments[0]](componentObject[arguments[0]]() ? false : true);
                    componentObject.firePropertiesChanged([arguments[0]]);
                }.bind(property.key)
            }
        } else { 
        }
    }
    return items;

}
VBIContextMenuDataLib.prototype.getDimensionDisplayItems = function() {
    var componentObject = this.parent.componentObject;
    var properties = [{
        name: "Default",
        value: "default"
    }, {
        name: "Key",
        value: "key"
    }, {
        name: "Text",
        value: "text"
    }, {
        name: "Key and Text",
        value: "key_text"
    }, {
        name: "Text and Key",
        value: "text_key"
    }]
    var items = {};
    if (!componentObject.dispkeytext) {
        return {}
    }
    var currentItem = componentObject.dispkeytext()
    for (var j = 0; j < properties.length; j++) {
        property = properties[j];
        items[property.value] = {
            name: property.name,
            icon: currentItem == property.value ? "fa-check" : "",
            callback: function() {
                componentObject.dispkeytext(arguments[0]);
                componentObject.firePropertiesChanged(['dispkeytext']);
            }.bind(property.value)

        }
    }
    return items;

}
VBIContextMenuDataLib.prototype.getExportItems = function() {
    var items = {};
    var self = this;
    var componentObject = this.parent.componentObject;
    var component = componentObject.$()
    var chart = isHighChart(component);
    if (!chart) {
        return {}
    }
    if (!chart.exportChart) {
        return {}
    }
    var exportOptions = {};
    items.exportPdf = {
        name: 'PDF',
        icon: 'fa-file-pdf-o',
        callback: function() {
            exportOptions.type = 'application/pdf'
            chart.exportChart(exportOptions);
        }
    }
    items.exportPng = {
        name: 'PNG Image',
        icon: 'fa-file-image-o',
        callback: function() {
            exportOptions.type = 'image/png'
            chart.exportChart(exportOptions);
        }
    }
    items.exportJpeg = {
        name: 'JPEG Image',
        icon: 'fa-file-image-o',
        callback: function() {
            exportOptions.type = 'image/jpeg'
            chart.exportChart(exportOptions);
        }
    }
    items.exportSvg = {
        name: 'SVG Image',
        icon: 'fa-file-image-o',
        callback: function() {
            exportOptions.type = 'image/svg+xml'
            chart.exportChart(exportOptions);
        }
    }
    return items;
}

function getGridOptionsDisplayTable(dataRows, R) {
    var header = dataRows[0];
    var data = _.rest(dataRows);
    var gridOptions = {
        rowSelection: 'multiple',
        enableColResize: true,
        enableSorting: true,
        enableFilter: true,
        enableRangeSelection: true,
        suppressRowClickSelection: true,
        animateRows: true,
        debug: false
    };

    var mapIndexed = R.addIndex(R.map);
    var headerKeyCount = {}
    var mapHeader = function(headerKey, index) {
        if (headerKey in headerKeyCount) {
            headerKeyCount[headerKey] = headerKeyCount[headerKey] + 1;
        } else {
            headerKeyCount[headerKey] = 0;
        }
        var isValue = false;
        var enableRowGroup = false;
        if (R.type(data[0][index]) == "Number") {
            isValue = true;
        } else {
            enableRowGroup = true;
        }
        header[index] = headerKey + headerKeyCount[headerKey];
        return {
            "headerName": headerKey,
            "field": headerKey + headerKeyCount[headerKey],
            enableValue: isValue,
            enableRowGroup: enableRowGroup,
            enablePivot: enableRowGroup
        }
    };
    gridOptions.columnDefs = mapIndexed(mapHeader, header);
    var flattenValue = function(dataObject) {
        if (R.type(dataObject) === "Object") {
            if (name in dataObject)
                return dataObject.name
            else
                return dataObject[R.keys(dataObject)[0]];
        } else {
            return dataObject;
        }
    }
    var flattenArray = function(dataArray) {
        return R.map(flattenValue, dataArray)
    }
    data = R.map(flattenArray, data)
    var mapHeaderAndValue = function(headerArray, valueArray) {
        return R.zipObj(headerArray, valueArray)
    }
    var mapHeaderAndValueCurry = R.curry(mapHeaderAndValue)(header)
    gridOptions.rowData = R.map(mapHeaderAndValueCurry, data)
    return gridOptions;
}

VBIContextMenuDataLib.prototype.preProcessDataRows = function(dataRows, rows, R) {
    var header = _.first(dataRows);
    var data = _.rest(dataRows);
    header = R.drop(1, header);
    header = R.insertAll(0, rows, header);
    var processArray = function(dataArray) {
        var rowMembersText = dataArray[0].name;
        var rowMembers = [];
        if (!rowMembersText) {
            rowMembersText = dataArray[R.keys(dataArray)[0]];
        }
        if (rowMembersText) {
            rowMembers = rowMembersText.split("\\");
            if (rowMembers.length != rows.length) {
                rowMembers = rowMembersText.split("/");
            }
            if (rowMembers.length != rows.length) {
                throw "Display data preprocess error, invalid dataRows or rows";
            }
            dataArray = R.drop(1, dataArray);
            dataArray = R.insertAll(0, rowMembers, dataArray);
        }
        return dataArray;

    }
    if (rows.length > 1);
    data = R.map(processArray, data);
    return R.insert(0, header, data);
}
VBIContextMenuDataLib.prototype.displayData = function(self) {
    if (!self)
        self = this;
    var R = this.parent.R;
    var componentObject = self.parent.componentObject;
    var data = componentObject.dataRes;
    var rows = [];
    var measureText;
    if (data) {
        rows = data.dimensions;
        rows = _.where(data.dimensions, {
            "axis": "ROWS"
        });
        var measureDimension = _.where(data.dimensions, {
            "axis": "ROWS",
            "containsMeasures": true
        });
        if (measureDimension.length == 1) {
            measureText = measureDimension[0].text;
        }

    }

    var component = componentObject.$();
    var chart = isHighChart(component);
    if (!chart) {
        alert("Error in Creating Tabel");
        return;
    }
    var isPropertyExist = function(property) {
        return property in componentObject;
    }
    var hierarchyLabelProperties = ["enablehierarchylabels", "activatehierarchylabeling"];
    var hierarchyLabelProperty = _.find(hierarchyLabelProperties, isPropertyExist);
    if (componentObject[hierarchyLabelProperty]) {
        if (componentObject[hierarchyLabelProperty]()) {
            componentObject.displayTable = true;
            componentObject.contextHierarchyLabel = true;
            componentObject[hierarchyLabelProperty](false);
            componentObject.firePropertiesChanged([hierarchyLabelProperty]);
            return;
        }
    }

    var tableLayer = $('<div class="vbi-context-table-layer"></div>');
    var tableContainer = $('<div class="vbi-context-table-container ag-fresh"  style="height: 90%;"></div>');
    var inputContainer = $('<div class="vbi-context-table-container"></div>')
    var quickFilterInput = $('<input type="text" id="quick-filter-input" placeholder="Type text to filter..."/>');
    var clearButton = $('<button id="clear-button">Clear</button>');
    var closeButton = $('<div class="vbi-close-button"><i class="fa fa-close vbi-minimize-chart" aria-hidden="true"></i></div>');
    var dataRows = chart.getDataRows();
    if (rows.length > 0) {
        rows = _.pluck(rows, "text");
        var measureIndex = measureText ? rows.indexOf(measureText) : 0;
        if (measureIndex != 0) {
            rows = R.remove(measureIndex, 1, rows);
            rows = R.insert(0, measureText, rows);
        }
        try {
            dataRows = self.preProcessDataRows(dataRows, rows, R);
        } catch (e) {
           if(typeof console != 'undefined') console.log(e);
        }
    }
    var gridOptions = getGridOptionsDisplayTable(dataRows, R);
    inputContainer.append(quickFilterInput).append(clearButton);
    tableLayer.append(inputContainer).append(tableContainer);
    component.append(tableLayer).append(closeButton);
    new agGrid.Grid(tableContainer[0], gridOptions);
    gridOptions.api.sizeColumnsToFit();
    componentObject.displayTable = true;
    quickFilterInput.on('keyup change', function() {
        gridOptions.api.setQuickFilter(quickFilterInput.val())
    });
    clearButton.on('click', function() {
        quickFilterInput.val("");
        gridOptions.api.setQuickFilter("");
    });
    self.parent.buttonContainer.hide();
    component.contextMenu(false);
    //table.DataTable({"lengthMenu": [[5,10, 25, 50, -1], [5,10, 25, 50, "All"]]});
    closeButton.on('click', function() {
         componentObject.displayTable = false;
        if (componentObject.contextHierarchyLabel) {
            componentObject.contextHierarchyLabel = false;
            componentObject[hierarchyLabelProperty](true);
            componentObject.firePropertiesChanged([hierarchyLabelProperty]);
        }
        self.parent.buttonContainer.show();
        closeButton.remove();
        tableLayer.remove();
        component.contextMenu(true);
    });

}
VBIContextMenuDataLib.prototype.getContextMenuItems = function(doNotFilterItems) {
    var self = this;
    var items;
    var selectedValues;
    var componentObject = self.parent.componentObject;
    var R = this.parent.R;
    var isSelectedValues = self.verifySelectedValues(componentObject);
    var onSelectContextItems = ["filter", "drillDown", "drillUp"];
    this.ztlCallRegister = new ZTLCallRegister(componentObject);
    items = {

        "displayData": {
            name: "Display Data",
            icon: "fa-table",
            callback: function() {
                self.displayData(self)
            }
        },
        "sort": {
            name: "Sort",
            icon: "fa-sort-amount-asc",
            items: self.getSortItems()
        },
        "filter": {
            name: "Filter",
            icon: "fa-filter",
            items: self.getDimensionFilterItems()
        },
        "ranking": {
            name: "Ranking",
            icon: "fa-sort-numeric-asc",
            items: self.getRankingItems()
        },
        "drillDown": {
            name: "Drill Down",
            icon: "fa-level-down",
            items: self.getDrillDownItems()
        },
        "drillUp": {
            name: "Drill Up",
            icon: "fa-level-up",
            items: self.getDrillUpItems()
        },
        "swapWith": {
            name: "Swap With",
            icon: "fa-arrows-h",
            items: self.getSwapWithDimensions()
        },
        "dimensionDisplay": {
            name: "Dimension Display",
            icon: "fa-list-ol",
            items: self.getDimensionDisplayItems()
        },
        "selectMeasure": {
            name: "Select Measure",
            icon: "fa-bars",
            items: self.getMeasureFilterItems()
        },
        "showHide": {
            name: "Show / Hide",
            icon: "fa-low-vision",
            items: self.getComponentPropertiesItems([{
                "key": "titleenable",
                'text': "Title"
            }, {
                "key": "subtitleenable",
                "text": "SubTitle"
            }, {
                "key": "legendenable",
                "text": "Legend"
            }, {
                "key": "tooltipenable",
                "text": "Tooltip"
            }, {
                "key": "datalabelenabled",
                "text": "Data Label"
            }, {
                "key": "xaxistitleenabled",
                "text": "X Axis Title"
            }, {
                "key": "yaxistitleenabled",
                "text": "Y Axis Title"
            }])

        },
        "totals": {
            name: "Totals",
            icon: "fa-th-large",
            items: self.getComponentPropertiesItems([{
                'key': 'showtotal',
                'text': 'Total'
            }, {
                'key': 'showsubtotal',
                'text': 'Sub Total'
            }])
        },
        "exportAs": {
            name: "Export As",
            icon: "fa-download",
            items: self.getExportItems()
        },
        "lassoItems": {
            name: "Lasso / Zoom",
            icon: "fa-adjust",
            items: self.getLassoItems()
        },
        "conditionalFormatting": {
            name: "Conditional Formatting",
            icon: "fa-pencil-square-o",
            items: self.getConditionalFormatingItmes(self)
        }

    };
    var componetType = componentObject.CID.toLowerCase().replace("chart", "");
    var componetItemsExcludeMap = {
        "sparkline": ["displayData", "lassoItems", "exportAs"],
        "marimekko": ["lassoItems"],
        "supercombination": ["totals", "displayData"],
        "heatmap": ["totals", "lassoItems", "conditionalFormatting"],
        "treemap": ["sort", "ranking", "lassoItems", "selectMeasure", "totals", "displayData"],
        "treemapdrilldown": ["sort", "ranking", "lassoItems", "selectMeasure", "totals", "displayData","filter","drillDown","drillUp","conditionalFormatting"],
        "polar": ["sort","lassoItems"],
        "groupstackedcolumnbar": ["sort"],
        "donut": ["sort","lassoItems"],
        "semicircledonut": ["sort"],
        "funneldd": ["sort","ranking","filter"],
        "funnelpyramid": ["sort"],
        "pie": ["sort"],
        "piedd": ["sort","ranking","filter"],
        "scatter": ["sort","displayData"],
        "bubble": ["sort","displayData"],
        "boxplot": ["sort","totals"],
        "radar": ["sort","lassoItems"],
        "waterfall":["totals"],
        "columnbardd":["ranking","filter"]
    };
    var noSubtotalCharts=["dualaxes","marimekko","multipleaxes"]
    if(noSubtotalCharts.indexOf(componetType)!=-1){
        if(items.totals.items){
            if(items.totals.items.showsubtotal){
                delete items.totals.items.showsubtotal;
            }
        }
    }
    if(componetType=="piedd"){
        items.selectMeasure.items=self.getMeasureFilterItemsPieFamily();
    }
    if (componetItemsExcludeMap[componetType]) {
        items = R.omit(componetItemsExcludeMap[componetType], items);
    }
    this.ztlCallRegister.postProcess();
    var isItemNotEmpty = function(object) {
        if (!object.items) {
            return true;
        }
        if (_.keys(object.items).length == 0) {
            return false;
        } else {
            return true;
        }
    }
    if (!self.parent.selectedKeys) {
        self.parent.selectedKeys = _.keys(items);
    }
    if (doNotFilterItems) {
        return items;
    }
    items = R.filter(isItemNotEmpty, items)

    var itemKeys = _.keys(items);
    var notSelectedItems = _.filter(itemKeys, function(key) {
        if (self.parent.selectedKeys.indexOf(key) != -1) {
            if (isSelectedValues) {
                return onSelectContextItems.indexOf(key) === -1
            } else {
                return false;
            }
        }
        return true;
    });
    return R.omit(notSelectedItems, items);


}

VBIContextMenuDataLib.prototype.verifySelectedValues = function(componentObject) {
    var componetType = componentObject.CID.toLowerCase().replace("chart", "");
    var drillDownCharts=["treemapdrilldown","piedd","funneldd"];
    if(drillDownCharts.indexOf(componetType)!=-1){
        return false;
    }
    var self = this;
    if (!componentObject.selectedvalues) {
        return false;
    }
    var selectedValues = componentObject.selectedvalues();
    if (!selectedValues) {
        return false;
    }
    try {
        selectedValues = JSON.parse("[" + selectedValues + "]");
    } catch (e) {
        return false;
    }
    var dimensions = componentObject.fulljson().dimensions;
    var rowDimensionKeys = self.getRowDimension(dimensions);
    var selectedValuesMap = self.getSelectedDimensionValues(selectedValues, rowDimensionKeys);
    if (_.isEmpty(selectedValuesMap)) {
        return false;
    }
    return true;
}