 /*Creates an instance of Flow Layer.
  *
  * @author Shabala Natarajan
  * @constructor
  * @param {map, props} Map Instance and Addt prop object..
  * @this {FlowLayer}
  */


 /* global  L */
 /* global LocDS */

 function FlowLayer(map, props) {
     var allLayers = {};
     var mapContainer = map;
     var flowData = {};
     var util = new VbiUtil();

     this.getFlowLayerConfig = function() {
         if (!_.isEmpty(props.flowLayerConfig)) {
             allLayers = JSON.parse(props.flowLayerConfig);
             return allLayers;
         } else {
             return false;
         }
     };

     this.getMapContainer = function() {
         return mapContainer;
     };

     this.getProps = function() {
         return props;
     };

     this.getUtil = function() {
         return util;
     };

     this.setAllLayers = function(f, graph) {
         allLayers[f] = graph;
     };

     this.buildFlowData = function(DS, flowConfig) {
         var flowLayerConfig = JSON.parse(flowConfig); // jshint ignore:line

         return flowData;

     };
 }

 function normalize(value, array) {
     var min = Math.max.apply(null, array);
     var max = Math.min.apply(null, array);
     if (max === min) {
         return 0;
     } else {
         value = (value - min) / (max - min);
         return value;
     }
 }

 function splitIntoBins(arr, n) {
     var rest = arr.length % n, // how much to divide
         restUsed = rest, // to keep track of the division over the elements
         partLength = Math.floor(arr.length / n),
         result = [];

     for (var i = 0; i < arr.length; i += partLength) {
         var end = partLength + i,
             add = false;
         if (rest !== 0 && restUsed) { // should add one element for the division
             end++;
             restUsed--; // we've used one division element now
             add = true;
         }
         result.push(arr.slice(i, end)); // part of the array
         if (add) {
             i++; // also increment i in the case we added an extra element for division
         }
     }
     return result;
 }

 function buildLegendWeight(component, config, weightArray) {
     var weights = [],
         ranges = [];
     var weightType = config.lineWeight.type;
     switch (weightType) {

         case 'pixel':
             _.each(weightArray, function(w) {
                 var minWeight = Math.min.apply(null, weightArray);
                 var maxWeight = Math.max.apply(null, weightArray);
                 var minPixel = parseFloat(config.lineWeight.min);
                 var maxPixel = parseFloat(config.lineWeight.max);
                 var difference = (maxPixel - minPixel) / (maxWeight - minWeight);
                 weights.push(((w - minWeight) * difference) + minPixel);
             });

             break;

         case 'percentage':
             _.each(weightArray, function(w) {
                 var fullScreenValue = Math.abs(component.$().height() * component.$().width());
                 var minPercent = parseFloat(config.lineWeight.min) * (fullScreenValue / 100);
                 var maxPercent = parseFloat(config.lineWeight.max) * (fullScreenValue / 100);
                 var minPixel = Math.sqrt(minPercent * 0.32);
                 var maxPixel = Math.sqrt(maxPercent * 0.32);
                 var difference = (maxPixel - minPixel) / (maxPercent - minPercent);
                 weights.push((((w - minPercent) * difference) + minPixel) / 10);
             });
             break;
     }
     weights = _.uniq(weights);
     weights = _.sortBy(weights, function(num) { return num; });
     weights = splitIntoBins(weights, 5);
     _.each(weights, function(w, i) {
         var len = w.length;
         if (i !== weights.length - 1) {
             ranges.push(w[0]);
         } else {
             ranges.push(w[len - 1]);
         }
     });
     return ranges;


 }

 function buildLegendColors(component, config) {
     var type = config.colorConfig;
     var colors = [];
     switch (type) {
         case 'dynamic':
             var dynamicColor = config.color;
             for (var dc in dynamicColor) {
                 colors.push(dynamicColor[dc].color);
             }
             break;

         case 'static':
             var userColor = config.color;
             for (var key in userColor) {
                 colors.push(userColor[key].color);
             }
             break;

         case 'brewer':
             var brewerColor = config.brewerColor;
             var colorScale;
             if (config.natureOfData != 'stan') {
                 colorScale = chroma.scale(brewerColor).colors(_.parseInt(config.buckets)); // jshint ignore:line
             } else if (config.natureOfData === 'stan') {
                 colorScale = chroma.scale(component.standardColor[brewerColor]).colors(_.parseInt(config.buckets)); // jshint ignore:line
             }
             colors = colorScale;
             break;
         default:
             break;
     }
     return colors;
 }

 function buildLegendValues(colors, values, config, legend) {
     var ranges = [];
     var count = colors.length;
     values = _.uniq(_.sortBy(values, function(num) { return num; }));
     if (legend === 'color') {
         var type = config.colorConfig;
         switch (type) {
             case 'dynamic':
                 var len = _.keys(config.color).length;
                 var max = Math.max.apply(null, values);
                 var range;
                 for (var i = 0; i < len; i++) {
                     range = (config.color[i].start * max / 100) + " - " + (config.color[i].stop * max / 100);
                     ranges.push(range);
                 }
                 break;
             case 'static':
                 var len = _.keys(config.color).length;
                 var range;
                 for (var i = 0; i < len; i++) {
                     range = config.color[i].start + " - " + config.color[i].stop;
                     ranges.push(range);
                 }
                 break;
             case 'brewer':
                 var bins = splitIntoBins(values, count);
                 var ranges = [];
                 _.each(bins, function(bin) {
                     var range;
                     if (bin.length === 1) {
                         range = bin[0];
                     } else {
                         range = bin[0] + " - " + bin[bin.length - 1];
                     }
                     ranges.push(range);
                 });
                 break;

             default:
                 break;

         }
     } else if (legend === 'weight') {
         var bins = splitIntoBins(values, count);
         var ranges = [];
         _.each(bins, function(bin) {
             var range;
             if (bin.length === 1) {
                 range = bin[0];
             } else {
                 range = bin[0] + " - " + bin[bin.length - 1];
             }
             ranges.push(range);
         });

     }
     return ranges;
 }


 function buildValues(markers, values) {

     var markerArray = [];
     values = _.uniq(values);
     var mkr = splitIntoBins(markers, values.length);
     for (var i = 0; i < mkr.length; i++) {
         if (i != (mkr.length - 1)) {
             markerArray.push(mkr[i][0]);
         } else {
             var m = mkr[i].length;
             markerArray.push(mkr[i][m - 1]);
         }
     }
     return markerArray;
 }

 function findWeights(flows, index) {
     var weights = [];
     for (var i = 0; i < flows.length; i++) {
         if (!_.isUndefined(flows[i][index])) {
             weights.push(parseFloat(flows[i][index]));
         } else {
             weights.push(0);
         }
     }
     return weights;
 }

 function findColor(component, config, colorArray, value) { //component = self.getProps();
     var type = config.colorConfig;
     var color;
     switch (type) {
         case 'dynamic':
             var dynamicColor = config.color;
             var measureValue = value;
             var minValue = Math.min.apply(null, colorArray);
             var maxValue = Math.max.apply(null, colorArray);
             var measurePercentage = ((measureValue - minValue) / (maxValue - minValue)) * 100;
             for (var key in dynamicColor) {
                 if (measurePercentage >= (dynamicColor[key].start) && measurePercentage <= (dynamicColor[key].stop)) {
                     color = dynamicColor[key].color;
                 }
             }
             break;

         case 'static':
             var userColor = config.color;
             var measureValue = value;
             for (var key in userColor) {
                 if (measureValue >= userColor[key].start && measureValue <= userColor[key].stop) {
                     color = userColor[key].color;
                 }
             }
             break;

         case 'brewer':
             var brewerColor = config.brewerColor;
             var colorScale;
             if (config.natureOfData != 'stan') {
                 colorScale = chroma.scale(brewerColor).colors(_.parseInt(config.buckets)); // jshint ignore:line
             } else if (config.natureOfData === 'stan') {
                 colorScale = chroma.scale(component.standardColor[brewerColor]).colors(_.parseInt(config.buckets)); // jshint ignore:line
             }
             var scale = d3.scale.quantile().domain(colorArray).range(colorScale); // jshint ignore:line
             color = scale(value);
             break;
         default:
             break;
     }
     return color;
 }

 function findOpacity(component, config, colorArray, value) { //component = self.getProps();
     var type = config.colorConfig;
     var opacity;
     switch (type) {
         case 'dynamic':
             var dynamicColor = config.color;
             var measureValue = value;
             var minValue = Math.min.apply(null, colorArray);
             var maxValue = Math.max.apply(null, colorArray);
             var measurePercentage = ((measureValue - minValue) / (maxValue - minValue)) * 100;
             for (var key in dynamicColor) {
                 if (measurePercentage >= (dynamicColor[key].start) && measurePercentage <= (dynamicColor[key].stop)) {
                     opacity = (dynamicColor[key].opacity);
                 }
             }
             break;

         case 'static':
             var userColor = config.color;
             var measureValue = value;
             for (var key in userColor) {
                 if (measureValue >= userColor[key].start && measureValue <= userColor[key].stop) {
                     opacity = userColor[key].opacity;
                 }
             }
             break;

         case 'brewer':
             opacity = 1;
             break;
     }
     return opacity;
 }

 function findWidth(component, config, weightArray, value) {
     var measureValue = value;
     var weightType = config.lineWeight.type;
     var weight;

     switch (weightType) {
         case 'pixel':
             var minWeight = Math.min.apply(null, weightArray);
             var maxWeight = Math.max.apply(null, weightArray);
             var minPixel = parseFloat(config.lineWeight.min);
             var maxPixel = parseFloat(config.lineWeight.max);
             var difference = (maxPixel - minPixel) / (maxWeight - minWeight);
             weight = ((measureValue - minWeight) * difference) + minPixel;
             break;

         case 'percentage':
             var fullScreenValue = Math.abs(component.$().height() * component.$().width());
             var minWeight = Math.min.apply(null, weightArray);
             var maxWeight = Math.max.apply(null, weightArray);

             var minPercent = parseFloat(config.lineWeight.min) * (fullScreenValue / 100);
             var maxPercent = parseFloat(config.lineWeight.max) * (fullScreenValue / 100);

             var minPixel = Math.sqrt(minPercent * 0.32);
             var maxPixel = Math.sqrt(maxPercent * 0.32);
             var difference = (maxPixel - minPixel) / (maxWeight - minWeight);
             weight = (((measureValue - minWeight) * difference) + minPixel) / 10;
             break;

         default:
             weight = 1;
             break;
     }
     return weight;

 }

 function constructToolTip(flow, config, ds) {
     var toolTipText;

     var primaryMeasure = (config.primaryMeasure === 'Choose Measure') ? 'N.A.' : flow.primaryMeasure;
     var secondaryMeasure = (config.secondaryMeasure === 'Choose Measure') ? 'N.A.' : flow.secondaryMeasure;
     toolTipText = 'Source: [' + flow.source[0] + ',' + flow.source[1] + '], Destination: [' + flow.destination[0] + ',' + flow.destination[1] + '], ';
     toolTipText = toolTipText + ds[config.primaryMeasure] + " : " + primaryMeasure;
     if (config.secondaryMeasure != 'Choose Measure') {
         toolTipText = toolTipText + ", " + ds[config.secondaryMeasure] + " : " + secondaryMeasure;
     }
     return toolTipText;
 }


 FlowLayer.prototype.deleteLayer = function(layer) {
     var self = this;
     var component = self.getProps();
     if (layer) {
         var id = L.Util.stamp(layer);
         var obj = self.getMapContainer()._layers[id];
         if (obj && self.getMapContainer().hasLayer(obj)) {
             self.getMapContainer().removeLayer(obj);
             component.layerControl.removeLayer(layer);
             delete self.getMapContainer()._layers[id];
         }
     }
 };


 FlowLayer.prototype.addFlowLayer = function(flowConfig) {
     var scripting = new Scripting(); // jshint ignore:line
     var self = this;
     var map = self.getMapContainer();
     var component = self.getProps();
     var config, datasource, flowObjects, flows, dims;
     var dataProviders = {},
         legend = {};
     datasource = LocDS.getInstance();

     var dataProvider = component.isDataProvider();
     //Check if -> Flow Layer is activated (useFlow) and proceed to adding layers
     //else -> delete all layers and set backUpFlowLayer to null.
     if (typeof component.useFlow() === 'boolean' && component.useFlow() && component.flowLayerConfig()) {
         config = JSON.parse(component.flowLayerConfig());
         //delete available layer in case of changes and add it afresh: Validate, Delete, Add functionality
         if (component.backUpFlowLayer.flowLayers && !_.isEqual(component.backUpFlowLayer.flowLayers, config)) {
                 var backUp = component.backUpFlowLayer.flowLayers;
                 for (var flow in backUp) {
                     if (!_.isEqual(backUp[flow], config[flow])) {
                         self.deleteLayer(component.flowReference[flow]);
                         delete component.flowReference[flow];
                     }
                 }
         }

         //Remove datasource layers in case of dataprovider (and vice-versa)
         for (var flow in config) {

             if ((!component.isDataProvider() && config[flow].isDataProvider) || (component.isDataProvider() && !config[flow].isDataProvider)) {
                 self.deleteLayer(component.flowReference[flow]);
                 delete component.flowReference[flow];
                 delete config[flow];
             }
         }

         //Internal function for rendering flow layer on map
         var renderFlowLayer = function(flow, currentResultSet) {
            
             legend[flow] = {};
             var graph = [];
             var colorArray, weightArray;
             var normalizedColorArray = [],
                 normalizedWeightArray = [];
             var hasFilter = (config[flow]['fDimension'] === 'No Filter') ? false : true;
             var fDimension;
             if (hasFilter) {
                 fDimension = dims[config[flow]['fDimension']];
                 for (var d in dims) {
                     if (dims[d] === fDimension) {
                         fDimension = d;
                     }
                 }
             }
             var filters = [];

             if (hasFilter) {
                 _.each(config[flow].filter, function(f) {
                     filters.push(f);
                 });
             }



             if (flowConfig[flow].primaryMeasure === "Choose Measure") {
                 colorArray = [];
                 normalizedColorArray = [];
             } else {
                 colorArray = findWeights(flows[flow], "primaryMeasure");
                 for (var c in colorArray) {
                     normalizedColorArray.push(normalize(colorArray[c], colorArray));
                 }
             }
             if (flowConfig.secondaryMeasure === "Choose Measure") {
                 weightArray = [];
             } else {
                 weightArray = findWeights(flows[flow], "secondaryMeasure");
                 for (var w in weightArray) {
                     normalizedWeightArray.push(normalize(weightArray[w], weightArray));
                 }
             }
             legend[flow]["colorMeasure"] = dims[config[flow]["primaryMeasure"]];
             var colors = buildLegendColors(component, config[flow]);
             legend[flow]["colors"] = buildValues(colors, colorArray);
             legend[flow]["colorValues"] = buildLegendValues(legend[flow]["colors"], colorArray, config[flow], 'color');
             legend[flow]["weightMeasure"] = dims[config[flow]["secondaryMeasure"]];
             legend[flow]["weight"] = buildLegendWeight(component, config[flow], weightArray);
             legend[flow]["weightValues"] = buildLegendValues(legend[flow]["weight"], weightArray, config[flow], 'weight');
             try {
                 for (var i = 0; i < flows[flow].length; i++) {
                     // if (!hasFilter || (_.indexOf(filters, flowObjects[i][fDimension]) != -1)) {
                     var flowPoint = flows[flow][i];
                     if (flowPoint.source[0] && flowPoint.source[1] && flowPoint.destination[0] && flowPoint.destination[1]) {

                         var source = new L.LatLng(flowPoint.source[0], flowPoint.source[1]);
                         var destination = new L.LatLng(flowPoint.destination[0], flowPoint.destination[1]);
                         var latlng = [];
                         latlng.push(source);
                         latlng.push(destination);
                         var primaryMeasure = normalize(parseFloat(flowPoint['primaryMeasure']), colorArray); // jshint ignore:line
                         var secondaryMeasure = normalize(parseFloat(flowPoint['secondaryMeasure']), weightArray); // jshint ignore:line
                         var color = findColor(component, config[flow], colorArray, flowPoint['primaryMeasure']);
                         var opacity = findOpacity(component, config[flow], colorArray, flowPoint['primaryMeasure']);
                         var weight = findWidth(component, config[flow], weightArray, flowPoint['secondaryMeasure']);
                         var showArrow = config[flow].showArrow;
                         var options;
                         /* = {
                                                     distanceToHeight: new L.LinearFunction([0, 5], [5000, 2000]), //([0, 2], [20, 400]), //([0, 0], [(primaryMeasure * 2000), 400]),
                                                     mode: 'C',
                                                     weight: weight,
                                                     color: color,
                                                     opacity: opacity,
                                                     'self': component,
                                                     'index': i,
                                                     'locDS': datasource,
                                                     'cEvent': false,
                                                     'data': currentResultSet,
                                                     'source': flowPoint.source,
                                                     'destination': flowPoint.destination
                                                 };*/
                         if (component.browser) {
                             options = {
                                 distanceToHeight: new L.LinearFunction([0, 5], [5000, 2000]), //([0, 2], [20, 400]), //([0, 0], [(primaryMeasure * 2000), 400]),
                                 mode: 'C',
                                 weight: weight,
                                 color: color,
                                 opacity: opacity,
                                 'self': component,
                                 'index': i,
                                 'locDS': datasource,
                                 'cEvent': false,
                                 'data': currentResultSet,
                                 'source': flowPoint.source,
                                 'destination': flowPoint.destination
                             };
                         } else {
                             options = {
                                 distanceToHeight: new L.LinearFunction([0, 5], [5000, 2000]), //([0, 2], [20, 400]), //([0, 0], [(primaryMeasure * 2000), 400]),
                                 mode: 'C',
                                 weight: weight,
                                 color: color,
                                 opacity: opacity
                             };

                         }


                         var tooltip = constructToolTip(flowPoint, flowConfig[flow], dims);
                         //var popup = L.popup().setContent(tooltip);
                         var arcPL = new L.ArcedPolyline(latlng, options);
                         arcPL.bindPopup(tooltip, { autoPanPadding:  L.point(300,  200) });

                         arcPL.on('mouseover', function() {
                             //functionality to be added
                         });
                         arcPL.on('mouseout', function() {
                             //functionality to be added
                         });
                         graph.push(arcPL);

                         arcPL.on('click', function(mk) {
                             scripting.callDSScripting(mk); // jshint ignore:line
                         });

                         if (showArrow) {
                             var marker = new L.StarMarker(destination, {
                                 // numberOfSides: 3,
                                 rotation: 90.0,
                                 radius: weight,
                                 color: color,
                                 opacity: opacity,
                                 fill: true,
                                 fillOpacity: opacity,
                                 numberOfPoints: 3
                             });
                             graph.push(marker);

                         }
                     }
                     //}
                     component.layerVisibility[flow] = config[flow].isLoad;
                     component.leafletLayerStatus(JSON.stringify(component.layerVisibility));
                     component.firePropertiesChanged(['leafletLayerStatus']);
                 }

             } catch (e) {
                 // console.log(e);
             }

             filters = [];

             var flowGraph = new L.layerGroup(graph);
             if (_.isUndefined(component.flowReference[flow])) {
                 component.layerControl.addOverlay(flowGraph, flow);
                 component.flowReference[flow] = flowGraph;
                 map.addLayer(flowGraph); // jshint ignore:line   
                 component.isAllLayersNameLoaded = true; 
             }

             if (!config[flow].isLoad) {
                 $(this).removeClass('selected');
                 self.getMapContainer().removeLayer(flowGraph);
                 component.layerVisibility[flow] = false;
             }
         };

         //handling data acc to dataprovider/datasource
         if (dataProvider) {
             if (config && !_.isEmpty(config)) {
                 for (var lyr in config) {
                     if (config[lyr].dataSource) {
                         dataProviders[lyr] = config[lyr].dataSource;
                     }
                 }

                 for (var flow in dataProviders) {

                     if (dataProviders[flow] === 'Choose Datasource') {
                         return;
                     }
                     component.datasourceUsed.push(dataProviders[flow]); //Add dataprovider control id to datasourceUsed - Enabling dynamic filtering
                     var resultset = JSON.parse(component.getDataProviderResult(dataProviders[flow]));
                     flowObjects = datasource.getFlowObjects(resultset);
                     flows = datasource.flowData(flowObjects, component.flowLayerConfig());
                     dims = datasource.getDimensionAndMeasureObj(resultset);
                     renderFlowLayer(flow, resultset);
                 }
             }

         } else if (component.resultset()) { //if datasource is used
             if (!_.isEmpty(config)) {
                 flowObjects = datasource.getFlowObjects(component.resultset());
                 flows = datasource.flowData(flowObjects, component.flowLayerConfig());
                 dims = datasource.getDimensionAndMeasureObj(component.resultset());

                 for (var flow in config) {
                     renderFlowLayer(flow, component.resultset());
                 }
             }

         }

         //Configure and add legend
         if (!sap.zen.designmode) {
            if($('#markerLegends #flowLegendForm').length === 0){
             $('#markerLegends').append('<form class="leaflet-control-layers-list" id="flowLegendForm">');
             $('#flowLegendForm').append('<table class="table-striped" id="legendTable"></table>');
             for (var l in legend) {
                 if (l !== 'undefined' && !_.isUndefined(legend[l].colorMeasure)) {
                     $('#legendTable').append('<tr class=' + l + ' style=cursor:pointer><td><i class = lName id = ' + l + '>' + l + '</i></td></tr>');

                     if (legend[l]['weightMeasure']) {
                         var len = (legend[l].colorValues.length >= legend[l].weightValues.length) ? legend[l].colorValues.length : legend[l].weightValues.length;
                         $('#legendTable').append('<tr class=' + l + '><td>Color</td><td>' + legend[l]["colorMeasure"] + '</td><td>Width</td><td>' + legend[l]['weightMeasure'] + '</td></tr>');
                         for (var i = 0; i < len; i++) {
                             var clr, wt, clrVal, wtVal;
                             if (!legend[l].colors[i]) {
                                 clr = '';
                                 clrVal = '';
                                 wt = legend[l].weight[i];
                                 wtVal = legend[l].weightValues[i];
                             } else if (!legend[l].weight[i]) {
                                 wt = 0;
                                 wtVal = '';
                                 clr = legend[l].colors[i];
                                 clrVal = legend[l].colorValues[i];
                             } else {
                                 wt = legend[l].weight[i];
                                 wtVal = legend[l].weightValues[i];
                                 clr = legend[l].colors[i];
                                 clrVal = legend[l].colorValues[i];
                             }

                             $('#legendTable').append('<tr class=' + l + '><td><i style="background:' + clr + ';height: 18px;width: 8px;float: left;margin-left: 8px;margin-right: 5px;"></i></td><td>' + clrVal + '</td><td><i style="background: #A9A9A9;height: 18px;width: ' + wt + 'px;float: left;margin-left: 8px;margin-right: 5px;"></i></td><td>' + wtVal + '</td></tr>');
                         }
                     } else {
                         $('#legendTable').append('<tr class=' + l + '><td>Color</td><td>' + legend[l]["colorMeasure"] + '</td></tr>');
                         for (var i = 0; i < legend[l].colorValues.length; i++) {

                             $('#legendTable').append('<tr class=' + l + '><td><i style="background:' + legend[l].colors[i] + ';height: 18px;width: 8px;float: left;margin-left: 8px;margin-right: 5px;"></i></td><td>' + legend[l].colorValues[i] + '</td></tr>');
                         }
                     }
                 }
             }



             // Controlling show/hide flow legend
             $('.flowLayers').off('click', function() {});
             $('.flowLayers').on('click', function() {
                 var layerControl = {};
                 //Separating and organizing the layers
                 for (var lyr in component.layerControl._layers) {
                     layerControl[component.layerControl._layers[lyr].name] = component.layerControl._layers[lyr].layer;
                 }
                 event.stopImmediatePropagation();
                 if (self.getMapContainer().hasLayer(layerControl[this.id])) {
                     $('#' + this.id).css('background-color', 'lightgray');
                     $('#' + this.id).css('opacity', '0.4');
                     $(this).removeClass('selected');
                     self.getMapContainer().removeLayer(layerControl[this.id]);
                 } else {
                     $('#' + this.id).css('background-color', 'transparent');
                     $('#' + this.id).css('opacity', '1');
                     self.getMapContainer().addLayer(layerControl[this.id]);
                     $(this).addClass('selected');
                 }
             });
             $('.leaflet-flow-legend')
                 .mouseover(function() {
                     $(".leaflet-flow-legend").addClass("leaflet-control-layers-expanded");
                 });

             $('#fLegend').click(function() {
                 $(".leaflet-flow-legend").removeClass("leaflet-control-layers-expanded");
             });
         }
         }

         //Update backUpFlowLayer
         if (!_.isEqual(component.backUpFlowLayer.flowLayers, config)) {
             component.backUpFlowLayer.flowLayers = config;
         }

         if (component.backUpFlowLayer.activateFlow !== component.useFlow()) {
             component.backUpFlowLayer.activateFlow = component.useFlow();
         }

         if (component.backUpFlowLayer.dataProviderFlag !== component.isDataProvider()) {
             component.backUpFlowLayer.dataProviderFlag = component.isDataProvider();
         }

     } else if (component.flowLayerConfig()) {

         config = JSON.parse(component.flowLayerConfig());
         if (Object.keys(config).length > 0) {
             for (var flow in config) {
                 self.deleteLayer(component.flowReference[flow]);
                 delete component.flowReference[flow];
             }
             component.backUpFlowLayer.flowLayers = null;

         }
         if (component.backUpFlowLayer.activateFlow !== component.useFlow()) {
             component.backUpFlowLayer.activateFlow = component.useFlow();
         }
     }
 };

 //Funtion deprecated in current version
 /*FlowLayer.prototype.setLegendVisibility = function(controls) {
     var self = this;
     var component = self.getProps();
     if (_.isEmpty(component.flowLayerConfig())) {
         return;
     }
     var config = JSON.parse(component.flowLayerConfig());

     for (var c in controls) {

         if (config[$.trim(c)]) {
             if (controls[c] === false) {
                 c = $.trim(c);
                 $('.' + c).hide();
             } else {
                 c = $.trim(c);
                 $('.' + c).show();

             }
         }

     }

 };*/


 //Function deprecated in current version

 /*FlowLayer.prototype.removeFlowLayer = function() { //new flowGraph post removal of required layer

     var self = this;
     var component = self.getProps();
     self.getMapContainer().removeControl(component.layerControl);
     component.layerControl = L.control.layers();
     component.layerControl.addTo(self.getMapContainer());
     if (component.flowReference && component.flowReference.length > 0) {
         for (var lyr in component.flowReference) {
             var id = L.Util.stamp(component.flowReference[lyr]);

             var obj = self.getMapContainer()._layers[id];
             if (obj && self.getMapContainer().hasLayer(obj)) {

                 self.getMapContainer().removeLayer(obj);
                 delete self.getMapContainer()._layers[id];
             }
         }
     }

 };
 */