/**
 * Creates an instance of Lasso.
 *
 * @author Vignesh Gopalakrishnan
 * @constructor
 * @param {map, props} Map Instance and Addt prop object..
 * @this {Lasso}
 */

/* global L */
/* global md5 */
function Lasso(map, props) {
  var mapContainer = map;
  var addProps = props;
  this.getMapContainer = function() {
    return mapContainer;
  };

  this.getProps = function() {
    return addProps;
  };
}

Lasso.prototype.createLassoObject = function() {
  var self = this;
  var component = self.getProps();
  component.featureGroup = L.featureGroup();
  var options = {
      position: 'topleft',
      draw: {
          polyline: false,
          polygon: {
              allowIntersection: false, // Restricts shapes to simple polygons
              drawError: {
                  color: '#e1e100', // Color the shape will turn when intersects
                  message: '<strong>Oh snap!<strong> you can\'t draw that!' // Message that will show when intersect
              },
              shapeOptions: {
                  stroke: true,
                  color: '#f06eaa',
                  weight: 1,
                  opacity: 0.5,
                  fill: true,
                  fillColor: null, //same as color by default
                  fillOpacity: 0.2,
                  clickable: true
              }
          },
          circle: false, // Turns off this drawing tool
          rectangle: false,
          marker: false
      },
      edit: {
          featureGroup: component.featureGroup, //REQUIRED!!
          edit:true,
          remove: true
      }
  };
  if (component.lasso === null) {
    self.getMapContainer().addLayer(component.featureGroup);
	if(!L.Control.Draw){
		
		L.Control.Draw = L.Control.extend({

			// Options
			options: {
				position: 'topleft',
				draw: {},
				edit: false
			},

			// @method initialize(): void
			// Initializes draw control, toolbars from the options
			initialize: function (options) {
				if (L.version < '0.7') {
					throw new Error('Leaflet.draw 0.2.3+ requires Leaflet 0.7.0+. Download latest from https://github.com/Leaflet/Leaflet/');
				}

				L.Control.prototype.initialize.call(this, options);

				var toolbar;

				this._toolbars = {};

				// Initialize toolbars
				if (L.DrawToolbar && this.options.draw) {
					toolbar = new L.DrawToolbar(this.options.draw);

					this._toolbars[L.DrawToolbar.TYPE] = toolbar;

					// Listen for when toolbar is enabled
					this._toolbars[L.DrawToolbar.TYPE].on('enable', this._toolbarEnabled, this);
				}

				if (L.EditToolbar && this.options.edit) {
					toolbar = new L.EditToolbar(this.options.edit);

					this._toolbars[L.EditToolbar.TYPE] = toolbar;

					// Listen for when toolbar is enabled
					this._toolbars[L.EditToolbar.TYPE].on('enable', this._toolbarEnabled, this);
				}
				L.toolbar = this; //set global var for editing the toolbar
			},

			// @method onAdd(): container
			// Adds the toolbar container to the map
			onAdd: function (map) {
				var container = L.DomUtil.create('div', 'leaflet-draw'),
					addedTopClass = false,
					topClassName = 'leaflet-draw-toolbar-top',
					toolbarContainer;

				for (var toolbarId in this._toolbars) {
					if (this._toolbars.hasOwnProperty(toolbarId)) {
						toolbarContainer = this._toolbars[toolbarId].addToolbar(map);

						if (toolbarContainer) {
							// Add class to the first toolbar to remove the margin
							if (!addedTopClass) {
								if (!L.DomUtil.hasClass(toolbarContainer, topClassName)) {
									L.DomUtil.addClass(toolbarContainer.childNodes[0], topClassName);
								}
								addedTopClass = true;
							}

							container.appendChild(toolbarContainer);
						}
					}
				}

				return container;
			},

			// @method onRemove(): void
			// Removes the toolbars from the map toolbar container
			onRemove: function () {
				for (var toolbarId in this._toolbars) {
					if (this._toolbars.hasOwnProperty(toolbarId)) {
						this._toolbars[toolbarId].removeToolbar();
					}
				}
			},

			// @method setDrawingOptions(options): void
			// Sets options to all toolbar instances
			setDrawingOptions: function (options) {
				for (var toolbarId in this._toolbars) {
					if (this._toolbars[toolbarId] instanceof L.DrawToolbar) {
						this._toolbars[toolbarId].setOptions(options);
					}
				}
			},

			_toolbarEnabled: function (e) {
				var enabledToolbar = e.target;

				for (var toolbarId in this._toolbars) {
					if (this._toolbars[toolbarId] !== enabledToolbar) {
						this._toolbars[toolbarId].disable();
					}
				}
			}
		});

	}
    component.lasso = new L.Control.Draw(options);
    self.getMapContainer().addControl(component.lasso);
  } else {
    component.featureGroup.clearLayers();
    self.getMapContainer().removeControl(component.lasso);
    component.lasso = new L.Control.Draw(options);
    self.getMapContainer().addControl(component.lasso);
  }

  return self.lasso;
};
/**
 *
 *
 * @return {null} .
 */
Lasso.prototype.draw = function() {
  var self = this;
  var component = self.getProps();
  //var datasource = LocDS.getInstance();
  var dsRows = component.usedPoints;
  self.createLassoObject();
  //lasso selected markers
  var lassoSelected = [];
  self.getMapContainer().off('draw:deleted');
  self.getMapContainer().on('draw:deleted', function(e) {
   var layers = e.layers;
   layers.eachLayer(function (layer) {
    //Deleting all the layers from the map #leaflet1.0 update
    self.getMapContainer().removeLayer(layer);
    //All the selected boundries will be displayed here.
    var selectedBounds = layer.getLatLngs();
    selectedBounds = selectedBounds[0];
    //Minimal bounds is formed here
    var bounds = self.getBounds(selectedBounds);
    //Adding the bounds globaly with md5 hash for finding duplicates.
    delete component.lassoBound[md5(JSON.stringify(bounds))];
    //The array stores the points in the delted boundary
    var deletedData = [];
    //Iterating the selected objects to find the objects presents in the delted array
    for (var i in lassoSelected) {
      if (bounds.contains(new L.LatLng(lassoSelected[i].lat, lassoSelected[i].lng))) {
        deletedData.push(lassoSelected[i]);
      }
    }
    //Now removing the deletedData from the selected objects.
    for (var i in lassoSelected) {
      for (var j in deletedData) {
        if (deletedData[j].lat == lassoSelected[i].lat && deletedData[j].lng == lassoSelected[i].lng ) {
          //Changing the color back
          var selectedMarker = component.soloMarkerArr[md5(lassoSelected[i].lat+lassoSelected[i].lng+lassoSelected[i].name)];
          if(selectedMarker && selectedMarker.options.icon && selectedMarker.options.icon.options.oldColor){
              var changingColor = null;
              if(selectedMarker.options.type=='standard'){
                changingColor = L.SCNDesignStudioMarkers.icon({markerColor: selectedMarker.options.icon.options.oldColor,selectedColor: selectedMarker.options.icon.options.markerColor, iconSize: selectedMarker.options.icon.options.iconSize, anchorPosition: [0.5, 0.5]});
                selectedMarker.setIcon(changingColor);
              }else if(selectedMarker.options.type=='font'){
                changingColor= L.FontAwesomeMarker.icon({markerColor: selectedMarker.options.icon.options.oldColor,selectedColor: selectedMarker.options.icon.options.markerColor, iconSize: selectedMarker.options.icon.options.iconSize, anchorPosition: [0.5, 0.5],iconClass: selectedMarker.options.icon.options.iconClass, iconColor: selectedMarker.options.icon.options.iconColor});
                selectedMarker.setIcon(changingColor);
              }else if(selectedMarker.options.type=='symbol'){
                changingColor = L.AwesomeMarkers.icon({ icon: selectedMarker.options.icon.options.icon, markerColor: selectedMarker.options.icon.options.oldColor,selectedColor:selectedMarker.options.icon.options.markerColor, prefix: 'fa' });
                selectedMarker.setIcon(changingColor);
              }
          }else if(selectedMarker.options && selectedMarker.options.type=='bubble' && selectedMarker.options.oldColor){
            selectedMarker.setStyle({'fillColor':selectedMarker.options.oldColor});
            delete selectedMarker.options.oldColor;
          }
          lassoSelected.splice(i, 1);
        }
      }
    }
    //Commiting the objects.
    component.lassoSelected(JSON.stringify(lassoSelected));
    component.isLassoScriptingDone = true;
    component.firePropertiesChanged(['lassoSelected']);
   });
  });
  self.getMapContainer().off('draw:created');
  self.getMapContainer().on('draw:created', function(e) {
    var isReverseLasso = e.layer.options.reverseLasso;
    if(component.isChangeColorLasso()){
      e.layer.options.color= '#'+(Math.random()*0xFFFFFF<<0).toString(16);
    }
    
    //Adding to map from leaflet version 1.0
    if(!isReverseLasso){
      component.featureGroup.addLayer(e.layer);
      self.getMapContainer().addLayer(e.layer);
    }
    //All the selected boundries will be displayed here.
    var selectedBounds = e.layer.getLatLngs();
    selectedBounds = selectedBounds[0];
    //Minimal bounds is formed here
    var bounds = self.getBounds(selectedBounds);
    //Adding the bounds globaly with md5 hash for finding duplicates.
    component.lassoBound[md5(JSON.stringify(bounds))] = {
      'bounds': bounds,
      'selectedBounds': selectedBounds
    };
    //Updating this objects for hide/show feature where adding and deleting points.
    component.lassoUpdatedUsedPoints = dsRows;
    //The latitude and longitude pints inside the minimal bounds are in this minimalDataset array.
    var minimalDataset = [];
    var completeDataset = [];
    //dsRows is a array which contains all the points used in the map. Here it will help us the configure the minimal dataset in the said bound.
    for (var i in dsRows) {
      completeDataset.push(dsRows[i]);
      if (bounds.contains(new L.LatLng(dsRows[i].lat, dsRows[i].lng))) {
        minimalDataset.push(dsRows[i]);
      }
    }

    for (var i in minimalDataset) {
      var currentPoint = new L.LatLng(minimalDataset[i].lat, minimalDataset[i].lng);
      var oddNodes = false;
      var k, j = selectedBounds.length - 1;
      for (k = 0; k < selectedBounds.length; k++) {
        //fetch adjucent points of the polygon
        var polyK = selectedBounds[k];
        var polyJ = selectedBounds[j];
        //check the intersections
        if (polyK.lng > currentPoint.lng != polyJ.lng > currentPoint.lng && currentPoint.lat < (polyJ.lat - polyK.lat) * (currentPoint.lng - polyK.lng) / (polyJ.lng - polyK.lng) + polyK.lat) {
          oddNodes = !oddNodes;
        }
        //switch between odd and even
        j = k;
      }

      var changingColor;
      var selectedMarker = null;
      selectedMarker = component.soloMarkerArr[md5(minimalDataset[i].lat+minimalDataset[i].lng+minimalDataset[i].name)];

      if (oddNodes) {
        //mouse point is inside the polygon
        if (component.layerVisibility[minimalDataset[i].name]) {
          //component.soloMarkerArr
          lassoSelected.push(minimalDataset[i]);
          if(selectedMarker){
            if(!isReverseLasso && selectedMarker.options.icon && !selectedMarker.options.icon.options.oldColor){
              changingColor = null;
              if(selectedMarker.options.type=='standard'){
                changingColor= L.SCNDesignStudioMarkers.icon({markerColor: selectedMarker.options.icon.options.selectedColor || 'blue', selectedColor: selectedMarker.options.icon.options.selectedColor, oldColor:selectedMarker.options.icon.options.markerColor, iconSize: selectedMarker.options.icon.options.iconSize, anchorPosition: [0.5, 0.5]});
                selectedMarker.setIcon(changingColor);
              }else if(selectedMarker.options.type=='font'){
                changingColor= L.FontAwesomeMarker.icon({markerColor: selectedMarker.options.icon.options.selectedColor || 'blue',selectedColor: selectedMarker.options.icon.options.selectedColor,oldColor: selectedMarker.options.icon.options.markerColor, iconSize: selectedMarker.options.icon.options.iconSize, anchorPosition: [0.5, 0.5],iconClass: selectedMarker.options.icon.options.iconClass, iconColor: selectedMarker.options.icon.options.iconColor});
                selectedMarker.setIcon(changingColor);
              }else if(selectedMarker.options.type=='symbol'){
                changingColor = L.AwesomeMarkers.icon({ icon: selectedMarker.options.icon.options.icon, markerColor: selectedMarker.options.icon.options.selectedColor,selectedColor: selectedMarker.options.icon.options.selectedColor, oldColor: selectedMarker.options.icon.options.markerColor, prefix: 'fa' });
                selectedMarker.setIcon(changingColor);
              }
            }else if(!isReverseLasso && selectedMarker.options && selectedMarker.options.type=='bubble' && !selectedMarker.options.oldColor){
              selectedMarker.options.oldColor = selectedMarker.options.fillColor;
              selectedMarker.setStyle({'fillColor':selectedMarker.options.selectedColor});
            }
          }
        }
      }else{
        if(isReverseLasso && selectedMarker && selectedMarker.options.icon ){
              changingColor = null;
              if(selectedMarker.options.type=='standard'){
                changingColor= L.SCNDesignStudioMarkers.icon({markerColor: selectedMarker.options.icon.options.selectedColor || 'blue', selectedColor: selectedMarker.options.icon.options.selectedColor, oldColor:selectedMarker.options.icon.options.markerColor, iconSize: selectedMarker.options.icon.options.iconSize, anchorPosition: [0.5, 0.5]});
                selectedMarker.setIcon(changingColor);
              }else if(selectedMarker.options.type=='font'){
                changingColor= L.FontAwesomeMarker.icon({markerColor: selectedMarker.options.icon.options.selectedColor || 'blue',selectedColor: selectedMarker.options.icon.options.selectedColor,oldColor: selectedMarker.options.icon.options.markerColor, iconSize: selectedMarker.options.icon.options.iconSize, anchorPosition: [0.5, 0.5],iconClass: selectedMarker.options.icon.options.iconClass, iconColor: selectedMarker.options.icon.options.iconColor});
                selectedMarker.setIcon(changingColor);
              }else if(selectedMarker.options.type=='symbol'){
                changingColor = L.AwesomeMarkers.icon({ icon: selectedMarker.options.icon.options.icon, markerColor: selectedMarker.options.icon.options.selectedColor,selectedColor: selectedMarker.options.icon.options.selectedColor, oldColor: selectedMarker.options.icon.options.markerColor, prefix: 'fa' });
                selectedMarker.setIcon(changingColor);
              }
          }else if(isReverseLasso && selectedMarker && selectedMarker.options && selectedMarker.options.type=='bubble'){
            selectedMarker.options.oldColor = selectedMarker.options.fillColor;
            selectedMarker.setStyle({'fillColor':selectedMarker.options.selectedColor});
          }
      }
    }
    var reverseLasso = null;
    //Subtracting the selected members from whole dataset which gives us reverseLasso
    if(isReverseLasso){
      reverseLasso = completeDataset.filter(function(val) {
        for (var i in lassoSelected) {
          if (val.key === lassoSelected[i].key) { return false; }
        }
        return true;
      });

      _.each(reverseLasso, function(value) {
        var selectedMarker = component.soloMarkerArr[md5(value.lat+value.lng+value.name)];
        changingColor = null;
        if(selectedMarker){
          if(selectedMarker.options.type=='standard'){
            changingColor= L.SCNDesignStudioMarkers.icon({markerColor: selectedMarker.options.icon.options.selectedColor || 'blue', selectedColor: selectedMarker.options.icon.options.selectedColor, oldColor:selectedMarker.options.icon.options.markerColor, iconSize: selectedMarker.options.icon.options.iconSize, anchorPosition: [0.5, 0.5]});
            selectedMarker.setIcon(changingColor);
          }else if(selectedMarker.options.type=='font'){
            changingColor= L.FontAwesomeMarker.icon({markerColor: selectedMarker.options.icon.options.selectedColor || 'blue',selectedColor: selectedMarker.options.icon.options.selectedColor,oldColor: selectedMarker.options.icon.options.markerColor, iconSize: selectedMarker.options.icon.options.iconSize, anchorPosition: [0.5, 0.5],iconClass: selectedMarker.options.icon.options.iconClass, iconColor: selectedMarker.options.icon.options.iconColor});
            selectedMarker.setIcon(changingColor);
          }else if(selectedMarker.options.type=='symbol'){
            changingColor = L.AwesomeMarkers.icon({ icon: selectedMarker.options.icon.options.icon, markerColor: selectedMarker.options.icon.options.selectedColor,selectedColor: selectedMarker.options.icon.options.selectedColor, oldColor: selectedMarker.options.icon.options.markerColor, prefix: 'fa' });
            selectedMarker.setIcon(changingColor);
          }else if(selectedMarker.options.type=='bubble'){
            selectedMarker.options.oldColor = selectedMarker.options.fillColor;
            selectedMarker.setStyle({'fillColor':selectedMarker.options.selectedColor});
          }
        }
      });
    }
    
    var lassoDataSelection = null;
    if(!isReverseLasso){ 
      lassoDataSelection = self.getSelectedLassoDataSelection(lassoSelected);
    }else if(isReverseLasso){
      lassoDataSelection = self.getSelectedLassoDataSelection(reverseLasso);
    }
    var selectedLassoProps = [];
      if(!isReverseLasso){
      _.each(lassoSelected, function(value) {
        selectedLassoProps.push(_.omit(value, ['dim', 'match','msr','name','text']));
      });
    }else if(isReverseLasso){
       _.each(reverseLasso, function(value) {
        selectedLassoProps.push(_.omit(value, ['dim', 'match','msr','name','text']));
      });
    }
    component.lassoDataSelection(JSON.stringify(lassoDataSelection));
    component.lassoSelected(JSON.stringify(selectedLassoProps));
    component.isLassoScriptingDone = true;
    component.firePropertiesChanged(['lassoSelected','lassoDataSelection']);
  });

  self.getMapContainer().off('draw:edited');
  self.getMapContainer().on('draw:edited', function(e) {

    var layers = e.layers;
    var editedLasso = [];
    for(var marker in component.soloMarkerArr){
      var selectedMarker = component.soloMarkerArr[marker];// jshint ignore:line
      if(selectedMarker && selectedMarker.options.icon && selectedMarker.options.icon.options.oldColor){
          var changingColor = null; // jshint ignore:line
            if(selectedMarker.options.type=='standard'){
              changingColor = L.SCNDesignStudioMarkers.icon({markerColor: selectedMarker.options.icon.options.oldColor,selectedColor: selectedMarker.options.icon.options.markerColor, iconSize: selectedMarker.options.icon.options.iconSize, anchorPosition: [0.5, 0.5]});
              selectedMarker.setIcon(changingColor);
            }else if(selectedMarker.options.type=='font'){
              changingColor= L.FontAwesomeMarker.icon({markerColor: selectedMarker.options.icon.options.oldColor,selectedColor: selectedMarker.options.icon.options.markerColor, iconSize: selectedMarker.options.icon.options.iconSize, anchorPosition: [0.5, 0.5],iconClass: selectedMarker.options.icon.options.iconClass, iconColor: selectedMarker.options.icon.options.iconColor});
              selectedMarker.setIcon(changingColor);
            }else if(selectedMarker.options.type=='symbol'){
              changingColor = L.AwesomeMarkers.icon({ icon: selectedMarker.options.icon.options.icon, markerColor: selectedMarker.options.icon.options.oldColor,selectedColor:selectedMarker.options.icon.options.markerColor, prefix: 'fa' });
              selectedMarker.setIcon(changingColor);
            }
      }else if(selectedMarker.options && selectedMarker.options.type=='bubble' && selectedMarker.options.oldColor){
        selectedMarker.setStyle({'fillColor':selectedMarker.options.oldColor});
        delete selectedMarker.options.oldColor;
      }
    }
    layers.eachLayer(function (layer) {
        //All the selected boundries will be displayed here.
        var selectedBounds = layer._latlngs;
        selectedBounds = selectedBounds[0];
        //Minimal bounds is formed here
        var bounds = self.getBounds(selectedBounds);
        //Adding the bounds globaly with md5 hash for finding duplicates.
        component.lassoBound[md5(JSON.stringify(bounds))] = {
          'bounds': bounds,
          'selectedBounds': selectedBounds
        };
        //Updating this objects for hide/show feature where adding and deleting points.
        component.lassoUpdatedUsedPoints = dsRows;
        //The latitude and longitude pints inside the minimal bounds are in this minimalDataset array.
        var minimalDataset = [];
        var completeDataset = [];
        //dsRows is a array which contains all the points used in the map. Here it will help us the configure the minimal dataset in the said bound.
        for (var i in dsRows) {
          completeDataset.push(dsRows[i]);
          if (bounds.contains(new L.LatLng(dsRows[i].lat, dsRows[i].lng))) {
            minimalDataset.push(dsRows[i]);
          }
        }

        
        for (var i in minimalDataset) {
          var currentPoint = new L.LatLng(minimalDataset[i].lat, minimalDataset[i].lng);
          var oddNodes = false;
          var k, j = selectedBounds.length - 1;
          for (k = 0; k < selectedBounds.length; k++) {
            //fetch adjucent points of the polygon
            var polyK = selectedBounds[k];
            var polyJ = selectedBounds[j];
            //check the intersections
            if (polyK.lng > currentPoint.lng != polyJ.lng > currentPoint.lng && currentPoint.lat < (polyJ.lat - polyK.lat) * (currentPoint.lng - polyK.lng) / (polyJ.lng - polyK.lng) + polyK.lat) {
              oddNodes = !oddNodes;
            }
            //switch between odd and even
            j = k;
          }

          var changingColor;
          var selectedMarker = component.soloMarkerArr[md5(minimalDataset[i].lat+minimalDataset[i].lng+minimalDataset[i].name)];


          
          if (oddNodes) {
            //mouse point is inside the polygon
            if (component.layerVisibility[minimalDataset[i].name]) {
              //component.soloMarkerArr
              
              editedLasso.push(minimalDataset[i]);
              if(selectedMarker){
                if(selectedMarker.options.icon && !selectedMarker.options.icon.options.oldColor){
                  if(selectedMarker.options.type=='standard'){
                    changingColor= L.SCNDesignStudioMarkers.icon({markerColor: selectedMarker.options.icon.options.selectedColor || 'blue', selectedColor: selectedMarker.options.icon.options.selectedColor, oldColor:selectedMarker.options.icon.options.markerColor, iconSize: selectedMarker.options.icon.options.iconSize, anchorPosition: [0.5, 0.5]});
                    selectedMarker.setIcon(changingColor);
                  }else if(selectedMarker.options.type=='font'){
                    changingColor= L.FontAwesomeMarker.icon({markerColor: selectedMarker.options.icon.options.selectedColor || 'blue', selectedColor: selectedMarker.options.icon.options.selectedColor, oldColor: selectedMarker.options.icon.options.markerColor, iconSize: selectedMarker.options.icon.options.iconSize, anchorPosition: [0.5, 0.5],iconClass: selectedMarker.options.icon.options.iconClass, iconColor: selectedMarker.options.icon.options.iconColor});
                    selectedMarker.setIcon(changingColor);
                  }else if(selectedMarker.options.type=='symbol'){
                    changingColor = L.AwesomeMarkers.icon({ icon: selectedMarker.options.icon.options.icon, markerColor: selectedMarker.options.icon.options.selectedColor, selectedColor: selectedMarker.options.icon.options.selectedColor, oldColor: selectedMarker.options.icon.options.markerColor, prefix: 'fa' });
                    selectedMarker.setIcon(changingColor);
                  }
                }else if(selectedMarker.options.type=='bubble' && !selectedMarker.options.oldColor){
                  selectedMarker.options.oldColor = selectedMarker.options.fillColor;
                  selectedMarker.setStyle({'fillColor':selectedMarker.options.selectedColor});
                }
              }
            }
          }
        }
      });
        var lassoDataSelection = null;
        
        lassoDataSelection = self.getSelectedLassoDataSelection(editedLasso);
        
        var selectedLassoProps = [];
        _.each(editedLasso, function(value) {
          selectedLassoProps.push(_.omit(value, ['dim', 'match','msr','name','text']));
        });
        
        
        component.lassoDataSelection(JSON.stringify(lassoDataSelection));
        component.lassoSelected(JSON.stringify(selectedLassoProps));
        component.isLassoScriptingDone = true;
        component.firePropertiesChanged(['lassoSelected','lassoDataSelection']);
  });
};

Lasso.prototype.getSelectedLassoDataSelection = function(lassoSelected) {
    var lasoSel = lassoSelected;
    var objKey = [];
    var dimensions;
    var measure;
    var ttip;
    for(var key in lasoSel){
      objKey.push(lasoSel[key].key);
      dimensions = lasoSel[key].dim;
      measure = lasoSel[key].msr;
      ttip = lasoSel[key].ttip;
    }
    var combinedKeys={};
    for(var key in objKey){
      for(var dim in objKey[key]){
        if(undefined===combinedKeys[dim]){
          combinedKeys[dim] = [];
        }
        combinedKeys[dim].push(objKey[key][dim]);
      }
    }

    for(var dim in combinedKeys){
      if(dimensions[dim]===undefined){
        delete combinedKeys[dim];
      }
    }

    if(ttip!="Dimension or Measure"){
      combinedKeys['(MEASURES_DIMENSION)']=ttip;
    }
    return combinedKeys;
};

Lasso.prototype.getSeletedPoints = function() {
  var self = this;
  var component = self.getProps();
  
  if (component.isLasso()) {
    component.isLasso(false);
    component.isLassoScriptingDone = true;
    component.firePropertiesChanged(['isLasso']);
  }
  return JSON.stringify(component.lassoSelected);
};

Lasso.prototype.getBounds = function(selectedBounds) {
  //Will store the latitude and longitude of the bounds with which we can draw a rectange.
  //that is minimising the area.
  var lats = [];
  var lngs = [];
  //Pusing all the lats and lngs into the array.
  for (var bnd = 0; bnd < selectedBounds.length; bnd++) {
    lats.push(selectedBounds[bnd].lat);
    lngs.push(selectedBounds[bnd].lng);
  }
  //The first and second point will be the north east and sounth west point.
  //Small rectangel will be drawn with this to cut down the minimum bounds.
  var firstPoint = new L.LatLng(Math.max.apply(Math, lats), Math.max.apply(Math, lngs));
  var secondPoint = new L.LatLng(Math.min.apply(Math, lats), Math.min.apply(Math, lngs));
  return L.latLngBounds(secondPoint, firstPoint);
};
