/**
 * Attach properties & method defined in args to seriesLibrary object
 *
 * @this Series Library
 * @param {object} args
 * @return {object} seriesLibrary
 * @author Nilesh Bhandari
 *
 */
/* global  sap, validateImplementMethod, isObjectEmpty */


function familyMultiAxes(args) { //jshint ignore:line
  var that = this;
  /*
     * *********************************************************************
     * -Template for defining definition in familySeries
     * -All function definition need to be within the scope of familyXYZ
     * -Each definition of property should be based on the validation of
     *  method validateImplementMethod(args, [[function name string]])
     *
     *  Example:
       *  if (validateImplementMethod(args, 'cds')) {
       *    <-- Actual Function definition goes here -->
       *    that.cds = function() {
       *
       *      // Your own implementation starts here either extend or
       *      // write complete new implementation
       *
       *      // var result = SeriesLibrary.prototype.cds.call(that);
       *      // some specific implementation
       *
       *    };
       *  }
     *
     * -If function definition is defined without validateImplementMethod
     *  then particular method is included by default in all components that
     *  include this file!
     * *********************************************************************
     */
  // alert('[[Executed familyColumn]]');
  /*
     * Functions defined in this file :
     * createSeriesData
     * globalgooglefont
     */
  if (validateImplementMethod(args, 'createCFData')) {
    that.createCFData = function(component, type) {
      var self = this;
      var completeJSONCF = component.dataRes;
      var includeResult = component.showtotaldatabase();
      /*
             * This block of code will map data to Dual Axes & Multiple Axes.
             */
      /*
             * Working: 1.Check Precondition & Proceed only if satisfy. 2. Generate
             * resultIndex[], feasibleColumn[], feasibleRow[] based on includeResult
             * Flag. 3. Check if DataUtilty is enabled. If yes go to 4.1 else 5.1.
             * 4.1. Check for dimension selected in data utility. If no selection
             * return. 4.2. Generate fixedColumnDimensions[] & fixedRowDimensions[] only for the
             * dimension selected in data utility. 4.3. Create columnMergeTuple[][] &
             * rowMergeTuple[][], a 2D array where subArray elements represent the
             * rows or columns to be aggregated into one row or column respectively.
             * Aggregate as information available in columnMergeTuple &
             * rowMergeTuple, generate series name, assign stack index and yAxis
             * index, and assign to seriesData. 5.1. Generate series name, assign
             * stack index, yaxis index and map data. 6. If the chart is Multiple
             * Axes, create yAxis object based on the yAxisLimit and add to
             * defaultValue
             */
      /*
             * Precondition to satisfy - a. Is Datasource availble. b. Is Measure
             * available. c. Is column available. d. Is feasible column available.
             */
      /*
             * Initiate variables - resultIndex[Length - No. of dimension, -1 if no
             * RESULT member else member index], feasibleColumn[Contains index of
             * column feasible], feasibleRow[Contains index of row feasible],
             * fixedColumnDimensions[Length - No. of dimension, -1 if dimension belong to row
             * else dimension index], fixedRowDimensions[Length - No. of dimension, -1 if
             * dimension belong to columns else dimension index]
             */
      /* Return null, if no data */
      if (isObjectEmpty(completeJSONCF)) {
        return null;
      }
      /* Infer metaData of each selected point in runtime */
      component.categoriesIndex = [];
      /* Refers to each series chart type */
      var innerChartType = component.dataseriestype().split(',');
      var nInnerChartType = innerChartType.length;
      /* Refers to each series mapping to n-th yAxis */
      var seriesYaxis = [];
      if (!(component.dataseriesyaxis() === undefined || component.dataseriesyaxis() === '')) {
        seriesYaxis = component.dataseriesyaxis().split(',');
      }
      /* Retains innerChartType & yAxis mapping of each series during filtering in runtime */
      var measureTypePair;
      if (!(component.measuretypepair() === undefined || component.measuretypepair() === '')) {
        measureTypePair = JSON.parse(component.measuretypepair());
      }

      var DMchange = false;
      var dimName = component.getDimensionAndMeasure();
      if (component.oldDimname === undefined || component.oldDimname === '') {
        component.oldDimname = dimName.toString();
      }

      if (component.oldDimname != dimName.toString()) {
        DMchange = true;
        if (sap.zen.designmode) {
          component.oldDimname = dimName.toString();
        }
      }
      /* Rules for conditional formatting */
      // @Conditional Formatting changes as per Amit's Requirements
      var measureInCFSeries = {};
      // After creation assign its data to 'component.measureInCFSeries'
      var categoryInCFSeries = {};
      // After creation assign its data to 'component.categoryInCFSeries'
      var columnInCFSeries = {};
      // After creation assign its data to 'component.columnInCFSeries'
      /* Data variables for defaultValue */
      var seriesData = [];
      var seriesCategory = [];
      var xaxisCategoryKey = [];
      var measureCategory = [];
      /* Reference to dataSource JSON */
      var tupleMatrixInJSON = completeJSONCF.tuples;
      var dataArrayInJSON = completeJSONCF.data;
      var dimensionsInJSON = completeJSONCF.dimensions;
      var columnsTupleInJSON = completeJSONCF['axis_columns'];
      var rowsTupleInJSON = completeJSONCF['axis_rows'];
      var nDimensions = completeJSONCF.dimensions.length;
      var nDimensionInColumns = columnsTupleInJSON[0].indexOf(-1) == -1 ? columnsTupleInJSON[0].length : columnsTupleInJSON[0].indexOf(-1);
      var nColumns = completeJSONCF.columnCount;
      var nRows = completeJSONCF.rowCount;
      /* Flag variables */
      var includeResult = component.showtotaldatabase();
      // If the result from data source is to include
      var dataUtilityEnabled = false;//component.datautility();
      var aggregateType = component.chartaggregation() || 'sum';
      var measureDimensionIndex = -1;
      // Index of measure dimension in dimension array of JSON
      /* working variables */
      var resultIndex = [];
      // Store RESULT member index, if any, for each dimension
      var fixedColumnDimensions = [];
      // dimension in columns, selected in data utility are marked 1 & others are -1
      var nFixedColumnDimensions = 0;
      var fixedRowDimensions = [];
      // dimension in rows, selected in data utility are marked 1 & others are -1
      var nFixedRowDimensions = 0;
      var feasibleColumn = [];
      // Stores feasible columns
      var feasibleRow = [];
      // Stores feasible rows
      var selectedColumn = [];
      // Stores selected columns
      var yAxisLimit = 0;
      // No. of Y-Axis to be generated
      var scalingfactor = [];
      var measureInRows = false;
      var mesIndex=0;
       var mesAt = "axis_columns";
      for (var i = 0; i < completeJSONCF.dimensions.length; i++) {
        if (completeJSONCF.dimensions[i].containsMeasures === true) {
          mesIndex = i;
          mesAt = completeJSONCF.dimensions[i].axis;
         break;
        }
      }
      if(mesAt === "ROWS"){
        mesAt = "axis_rows";
      }else{
        mesAt = "axis_columns";
      }
      for (var j = 0; j < completeJSONCF[mesAt].length; j++) {
            if (isNaN(completeJSONCF.dimensions[mesIndex].members[completeJSONCF[mesAt][j][mesIndex]].scalingFactor) || completeJSONCF.dimensions[mesIndex].members[completeJSONCF[mesAt][j][mesIndex]].scalingFactor === undefined) {
              scalingfactor.push(0);
            }else {
              scalingfactor.push(completeJSONCF.dimensions[mesIndex].members[completeJSONCF[mesAt][j][mesIndex]].scalingFactor);
            }
      }

      if (completeJSONCF.dimensions[mesIndex].axis == 'ROWS') {
        measureInRows = true;
      }
      /*
             * Initialisation Loop
             * -------------------
             *
             * Get measureDimensionIndex from the datasource; +[Identify index
             * of member with type RESULT in dimension]; +[Initialize array
             * value of fixedColumnDimensions & fixedRowDimensions to -1];
             * Note: fixedColumnDimensions & fixedRowDimensions will contains dimensions
             * that will be part of output series
             */
      for (var i = 0, measureDimensionIndexFlag = false; i < nDimensions; ++i) {
        /*
                 * Get index of measure dimension in measureDimensionIndex
                 */
        if (!measureDimensionIndexFlag) {
          if (dimensionsInJSON[i].containsMeasures !== undefined) {
            measureDimensionIndex = i;
            measureDimensionIndexFlag = true;
          }
        }
        /*
                 * Initialize fixedColumnDimensions & fixedRowDimensions Array to default value -1.
                 * -1 states not to include. Initially set all dimensions not to include. Later based
                 * on data utility selection include selected dimensions
                 */
        fixedColumnDimensions[i] = -1;
        fixedRowDimensions[i] = -1;
        resultIndex[i] = -1;
        /*
                 * Get the Result Member of each dimension and update the
                 * resultIndex Array with the index position of member with type 'RESULT' in the
                 * order Dimensions appear in the JSON
                 */
        for (var j = 0, l = dimensionsInJSON[i].members.length; j < l; ++j) {
          if (dimensionsInJSON[i].members[j].type == 'RESULT') {
            resultIndex[i] = j;
            break;
          }
        }
      }
      /**
             * Validation
             */
      /*
             * Return immediately if no Measure Dimension exists in the data source
             */
      if (measureDimensionIndex == -1) {
        // this.createSmokeScreen(component.$(),"No Measure Dimension. Place Measure Dimension in datasorce");
        if (window.console) {
          console.log('No Measure Dimension Exists in the Data Source - Returning');  // jshint ignore:line
        }

        return -1;
      }
      /*
             * Return immediately if no Columns exists in the selection
             */
      if (nColumns === 0) {
        // this.createSmokeScreen(component.$(),"No Columns Exists. Place Dimension in Column");
        if (window.console) {
          console.log('No Columns in DataSource - Returning');  // jshint ignore:line
        }

        return -2;
      }
      /*
             * Create two arrays [feasibleColumn, feasibleRow] each having list of
             * columns and rows index that will be included in the output. If
             * includeResult is true, include all the rows and columns in the arrays
             * else only push those rows & columns which are not the Result Row or
             * Column by referencing resultIndex array
             */
      if (!includeResult) {
        /*
                 * includeResult is false, So include only non-result columns & rows
                 * in the feasibleColumn & feasibleRow
                 */
        /*
                 * Looping into all columns
                 */
        for (var m = 0; m < nColumns; ++m) {
          /*
                     * Loop thru all dimensions that make up the current column and
                     * check each member if it is result or not
                     */
          var matchedFlag = false;
          for (var n = 0; n < nDimensionInColumns; ++n) {
            /*
                         * If current member at index n of current dimension at
                         * index m matches the n-th index of resultIndex then its a
                         * result index and break the loop after setting the flag
                         * and add that index to feasible array only if the flag is
                         * reset
                         */
            if (tupleMatrixInJSON[m][n] == resultIndex[n]) {
              matchedFlag = true;
              break;
            }
          }

          if (!matchedFlag) {
            feasibleColumn.push(m);
          }
        }
        /*
                 * Looping into all columns
                 */
        for (var m = 0; m < nRows; ++m) {
          /*
                     * Loop thru all dimensions that make up the current rowand
                     * check each member if it is result or not
                     */
          var matchedFlag = false;
          for (var n = nDimensionInColumns; n < nDimensions; ++n) {
            /*
                         * If current member at index n of current dimension at
                         * index m matches the n-th index of resultIndex then its a
                         * result index and break the loop after setting the flag
                         * and add that index to feasible array only if the flag is
                         * reset
                         */
            if (tupleMatrixInJSON[m * nColumns][n] == resultIndex[n]) {
              matchedFlag = true;
              break;
            }
          }

          if (!matchedFlag) {
            feasibleRow.push(m);
          }
        }
      } else {
        /*
                 * includeResult is true, So include all columns & rows in the
                 * feasibleColumn & feasibleRow
                 */
        for (var m = 0; m < nColumns; ++m) {
          feasibleColumn.push(m);
        }

        for (var m = 0; m < nRows; ++m) {
          feasibleRow.push(m);
        }
      }
      /*
             * Get rows & columns dimension from data utility selection
             */
      var utilityColumnDimension = [];
      var utilityRowDimensions = [];
      /*
             * NEW IMPLEMENTATION
             * If dataUtility is enabled get selected row and column dimension.
             * If disabled, include all dimensions from row and column in
             * utilityRowDimensions & utilityColumnDimensions
             */
      if (dataUtilityEnabled) {
        var selectionList = component.bardatacustomization();
        /*
                 * Return if no selection in data utility
                 */
        if (selectionList === '' || selectionList == ';') {
          // this.createSmokeScreen(component.$(),"Please select Dimension in Data Utility");
          if (window.console) {
            console.log('No Selection of Dimension in Data Utility - returning');  // jshint ignore:line
          }

          return -3;
        }

        selectionList = JSON.parse(selectionList);
        // parse string to JSON
        // Update utilityColumnDimension with column selection
        for (var i = 0, iLimit = selectionList.COLUMNS.length; i < iLimit; ++i) {
          utilityColumnDimension.push(selectionList.COLUMNS[i].index);
        }
        // Update utilityRowDimension with row selection
        for (var i = 0, iLimit = selectionList.ROWS.length; i < iLimit; ++i) {
          utilityRowDimensions.push(selectionList.ROWS[i].index);
        }
        // Update the value to 1 stating included in fixedColumnDimensions
        for (var i = 0, iLimit = utilityColumnDimension.length; i < iLimit; ++i) {
          fixedColumnDimensions[utilityColumnDimension[i]] = 1;
          ++nFixedColumnDimensions;
        }
        // Update the value to 1 stating included in fixedRowDimensions
        for (var i = 0, iLimit = utilityRowDimensions.length; i < iLimit; ++i) {
          fixedRowDimensions[utilityRowDimensions[i]] = 1;
          ++nFixedRowDimensions;
        }

        if (measureDimensionIndex < nDimensionInColumns) {
          fixedColumnDimensions[measureDimensionIndex] = 1;
          ++nFixedColumnDimensions;
        } else {
          fixedRowDimensions[measureDimensionIndex] = 1;
          ++nFixedRowDimensions;
        }
      } else {
        for (var i = 0, iLimit = nDimensionInColumns; i < iLimit; ++i) {
          fixedColumnDimensions[i] = 1;
          ++nFixedColumnDimensions;
        }

        for (var i = nDimensionInColumns, iLimit = nDimensions; i < iLimit; ++i) {
          fixedRowDimensions[i] = 1;
          ++nFixedRowDimensions;
        }
      }
      /*
             * Create columnMergeTuples array that will have the subArray which
             * should be merged into one column. LOGIC:[Feasible Column Tuples
             * having same values at index that are set in fixedColumnDimensions
             * needs to be grouped]
             */
      var columnMergeTuples = [];
      // This will contain column to be merged
      var visitedColumns = [];
      var columnMergeCount = 0;
      for (var i = 0; i < feasibleColumn.length; ++i) {
        // Get next column if this is already column is already checked
        if (visitedColumns[i] !== undefined) {
          continue;
        }
        // Set current column value to looping index column
        var currentColumn = columnsTupleInJSON[feasibleColumn[i]];
        // Loop through all the remaining columns for checking
        for (var j = i; j < feasibleColumn.length; ++j) {
          // Get next column if this is already column is already checked
          if (visitedColumns[j] !== undefined) {
            continue;
          }
          // Set comparing column value to looping index column which needs to tested against current column
          var comparingColumn = columnsTupleInJSON[feasibleColumn[j]];
          /*
                     * Match currentColumn values with comparingColumn values at
                     * index fixedColumnDimensions to test if they have
                     * identical values. If matched, add comparingColumn index to columnMergeTuples
                     */
          var matched = true;
          for (var k = 0; k < nDimensionInColumns; ++k) {
            if (fixedColumnDimensions[k] == 1) {
              if (currentColumn[k] != comparingColumn[k]) {
                matched = false;
                break;
              }
            }
          }

          if (matched) {
            // If new column aggregating group is undefined, define it
            if (columnMergeTuples[columnMergeCount] === undefined) {
              columnMergeTuples[columnMergeCount] = [];
            }
            // Add the column to this group
            columnMergeTuples[columnMergeCount].push(feasibleColumn[j]);
            // Set this column as visited to avoid checking later for other column aggregating groups
            visitedColumns[j] = true;
          }
        }
        // Incrementing column aggregating group index to next value
        ++columnMergeCount;
      }
      /*
             * Set yAxisLimit variable to limit the number yAxis allowed in
             * chart. For Dual Axis Maximum yAxis allowed is 2
             */
      if (type != 'dualAxes') {
        yAxisLimit = columnMergeTuples.length - 1;
      } else {
        // Magic Number 2 is for Max yAxis for Dual Axis
        if (columnMergeTuples.length > 2) {
          yAxisLimit = 1;
        } else {
          yAxisLimit = columnMergeTuples.length - 1;
        }
      }
      /*
             * NEW IMPLEMENTATION
             * Populate selectedColumn with columnMergeTuple
             */
      for (var i = 0, iLimit = columnMergeTuples.length; i < iLimit; ++i) {
        selectedColumn.push(columnMergeTuples[i][0]);
      }
      /*
             * For keeping track of stack index allocated in the chart
             */
      var stacking = {
        column: [],
        area: [],
        areaspline: []
      };
      /*
             * Create Series object for master JSON Object & append to
             * defaultValue in the end of all manipulation. Loop thru
             * selectedColumns to create list of series object for each series
             * selected
             */
      for (var i = 0, yIndex = 0, iLimit = columnMergeTuples.length; i < iLimit; ++i) {
        /*
                 * Create Name for the series to be called in legends
                 */
        var tempName = '';
        var tempNameAsKey = '';
        var tempKeyvalue = '';
        var tempKey = '';
        var tempConditionalFormattingKey = '';
        var tempMeasureKey = '';
        var measureDimString = '';
        // Current column
        var tempColumn = columnsTupleInJSON[selectedColumn[i]];
        for (var k = 0; k < tempColumn.length; ++k) {
          if (fixedColumnDimensions[k] != -1) {
            if (k === measureDimensionIndex) {
              tempMeasureKey = dimensionsInJSON[k].members[tempColumn[k]].key;
              measureDimString = k + ':' + tempColumn[k];
            }
            if (tempName !== '' && tempName !== ' ') {
              tempName = tempName + '-' + dimensionsInJSON[k].members[tempColumn[k]].text;// VBX141-7967|| dimensionsInJSON[k].members[tempColumn[k]].key;
              tempKey = tempKey + '-' + k + ':' + tempColumn[k];
              tempNameAsKey = tempNameAsKey + '/ ' + dimensionsInJSON[k].members[tempColumn[k]].key || dimensionsInJSON[k].members[tempColumn[k]].text;
              tempConditionalFormattingKey = tempConditionalFormattingKey + '-' + dimensionsInJSON[k].members[tempColumn[k]].key || dimensionsInJSON[k].members[tempColumn[k]].text;
            } else {
              tempName = dimensionsInJSON[k].members[tempColumn[k]].text;//VBX141-7967 || dimensionsInJSON[k].members[tempColumn[k]].key;
              tempKey = k + ':' + tempColumn[k];
              tempNameAsKey = dimensionsInJSON[k].members[tempColumn[k]].key || dimensionsInJSON[k].members[tempColumn[k]].text;
              tempKeyvalue = dimensionsInJSON[k].members[tempColumn[k]].key;
              tempConditionalFormattingKey = dimensionsInJSON[k].members[tempColumn[k]].key || dimensionsInJSON[k].members[tempColumn[k]].text;
            }
          }
        }
        // For Conditional Formatting
        if (tempMeasureKey !== '') {
          if (nFixedColumnDimensions == 1) {
            measureInCFSeries[tempMeasureKey] = i;
          } else {
            measureInCFSeries[tempMeasureKey] = measureInCFSeries[tempMeasureKey] || {};
            measureInCFSeries[tempMeasureKey][tempConditionalFormattingKey] = i;
          }
        }

        columnInCFSeries[tempConditionalFormattingKey] = i;
        var tempChartType = innerChartType[i % nInnerChartType].split('-');
        var tempStacking;
        // This contains info if the stacking is normal or percentage based
        var tempStack = i;
        var tempYaxis = seriesYaxis[i];
        // Retain old mapping in case filtering DS affects series order
        if (measureTypePair && measureTypePair[tempNameAsKey] !== undefined) {
          if (measureTypePair[tempNameAsKey].type !== undefined) {
            tempChartType = measureTypePair[tempNameAsKey].type.split('-');
          }

          if (measureTypePair[tempNameAsKey].yAxis !== undefined) {
            tempYaxis = measureTypePair[tempNameAsKey].yAxis;
          }
        }

        if (tempChartType.length != 1) {
          // This is excuted if chart type have additional info of chart type
          tempStacking = tempChartType[2];
          // index 2 contains information regarding stacking
          var innerStacking = tempStacking === undefined ? 0 : 1;
          if (tempStacking === undefined) {
            tempStacking = 'normal';
          }

          if (stacking[tempChartType[0]][innerStacking] === undefined) {
            tempYaxis = +(tempYaxis === undefined ? yIndex > yAxisLimit ? yAxisLimit : yIndex++ : tempYaxis);
            stacking[tempChartType[0]][innerStacking] = [];
            stacking[tempChartType[0]][innerStacking][0] = i;
            stacking[tempChartType[0]][innerStacking][1] = tempYaxis;  //_.parseInt(seriesYaxis[i] === undefined? (i > yAxisLimit? yAxisLimit : i) : seriesYaxis[i]);
          }

          tempStack = stacking[tempChartType[0]][innerStacking][0];
          tempYaxis = stacking[tempChartType[0]][innerStacking][1];
        } else {
          tempYaxis = +(tempYaxis === undefined ? yIndex > yAxisLimit ? yAxisLimit : yIndex++ : tempYaxis);
        }
        /*
                 * Assign default values if required and declare other objects
                 */
        seriesData[i] = {
          data: [],
          key: tempKey,
          name: self.formatSeriesName(component, tempName, measureDimString, {}),
          nameAsKey: tempNameAsKey,
          keyvalue: tempKeyvalue,
          uniquekey: tempKeyvalue,
          //required for errorbar to identify unique selection of series
          yAxis: tempYaxis,
          type: tempChartType[0],
          stack: tempStack,
          stacking: tempStacking
        };
      }
      /*
             * Create rowMergeTuples array that will have the subArray which
             * should be merged into one row. LOGIC:[Same as columnMergeTuple]
             */
      var visitedRows = [];
      var rowMergeTuples = [];
      var rowMergeCount = 0;
      for (var i = 0; i < feasibleRow.length; ++i) {
        if (visitedRows[i] !== undefined) {
          continue;
        }

        var currentRow = rowsTupleInJSON[feasibleRow[i]];
        for (var j = i; j < feasibleRow.length; ++j) {
          if (visitedRows[j] !== undefined) {
            continue;
          }

          var comparingRow = rowsTupleInJSON[feasibleRow[j]];
          var matched = true;
          for (var k = nDimensionInColumns; k < nDimensions; ++k) {
            if (fixedRowDimensions[k] == 1) {
              if (currentRow[k] != comparingRow[k]) {
                matched = false;
                break;
              }
            }
          }

          if (matched) {
            if (rowMergeTuples[rowMergeCount] === undefined) {
              rowMergeTuples[rowMergeCount] = [];
            }

            rowMergeTuples[rowMergeCount].push(feasibleRow[j]);
            visitedRows[j] = true;
          }
        }

        ++rowMergeCount;
      }
      /*
             * Now rowMergeTuple & columnMergeTuple Array contains the required
             * the merging rules. Loop through the list of sub-array of both
             * Array and merge sub-array as one row or column. Note for column
             * aggregation always it is sum
             */
      var aggregatedData = [];
      for (var i = 0; i < rowMergeTuples.length; ++i) {
        aggregatedData[i] = [];
        var tempName = '';
        var tempKey = '';
        var tempRow = rowsTupleInJSON[rowMergeTuples[i][0]];
        var dimensionMemberPair = '';
        var tempMeasureKey = '';
        var tempConditionalFormattingKey = '';
        for (var k = 0; k < tempRow.length; ++k) {
          if (fixedRowDimensions[k] != -1) {
            if (k == measureDimensionIndex) {
              tempMeasureKey = dimensionsInJSON[k].members[tempRow[k]].key;
            }

            if (tempName !== '' && tempName !== ' ') {
              if(dimensionsInJSON[k].members[tempRow[k]].text){
                tempName = tempName + '-' + dimensionsInJSON[k].members[tempRow[k]].text; //VBX141-5817|| dimensionsInJSON[k].members[tempRow[k]].key;
              }
              tempKey = tempKey + '*' + dimensionsInJSON[k].members[tempRow[k]].key;
              dimensionMemberPair = dimensionMemberPair + '-' + k + ':' + tempRow[k];
              // jscs:disable
              // Code here will be ignored by JSCS.
              tempConditionalFormattingKey = tempConditionalFormattingKey + '\\' + dimensionsInJSON[k].members[tempRow[k]].key || dimensionsInJSON[k].members[tempRow[k]].text;  // jscs:enable
            } else {
              tempName = dimensionsInJSON[k].members[tempRow[k]].text;//VBX141-5817 || dimensionsInJSON[k].members[tempRow[k]].key;
              dimensionMemberPair = k + ':' + tempRow[k];
              tempKey = dimensionsInJSON[k].members[tempRow[k]].key;
              tempConditionalFormattingKey = dimensionsInJSON[k].members[tempRow[k]].key || dimensionsInJSON[k].members[tempRow[k]].text;
            }
          }
        }
        // For Conditional Formatting
        if (tempMeasureKey !== '') {
          if (nFixedRowDimensions == 1) {
            measureInCFSeries[tempMeasureKey] = i;
          } else {
            measureInCFSeries[tempMeasureKey] = measureInCFSeries[tempMeasureKey] || {};
            measureInCFSeries[tempMeasureKey][tempConditionalFormattingKey] = i;
          }
        }

        categoryInCFSeries[tempConditionalFormattingKey] = i;
        component.categoriesIndex.push(dimensionMemberPair);
        seriesCategory.push(tempName);
        xaxisCategoryKey.push(tempKey);
        measureCategory.push(tempMeasureKey);
        var workingArray = [];
    var partialAggregation = null;
        for (var k = 0; k < selectedColumn.length; ++k) {
          workingArray[k] = [];
          for (var j = 0; j < rowMergeTuples[i].length; ++j) {
            var mergeColumns = columnMergeTuples[k];
            var mergeColumnData = [];
            for (var p = 0, pLimit = mergeColumns.length; p < pLimit; ++p) {
              var datavalue = dataArrayInJSON[+(rowMergeTuples[i][j] * nColumns) + +mergeColumns[p]];//jshint ignore:line
              if(datavalue === null){
                continue;
              } 
              if (measureInRows) {
                mergeColumnData.push(datavalue * Math.pow(10, scalingfactor[rowMergeTuples[i][j]]));   //jshint ignore:line
              }else if (!measureInRows) {
                mergeColumnData.push(datavalue * Math.pow(10, scalingfactor[mergeColumns[p]]));   //jshint ignore:line
              }

            }
          if(mergeColumnData.length === 0){
            continue;
           }           
            partialAggregation = this.aggregateArray(mergeColumnData, 'sum');
       
            workingArray[k].push(partialAggregation);
          }
        }

        for (var k = 0; k < selectedColumn.length; ++k) {
          if (workingArray[k] === ''  || workingArray[k].length === 0){
        aggregatedData[i][k] = null;
      }
          else{
        aggregatedData[i][k] = this.aggregateArray(workingArray[k], aggregateType);
      }

        }

        for (var k = 0; k < selectedColumn.length; ++k) {
          seriesData[k].data.push(aggregatedData[i][k]);
        }

      }
      // Assign required properties for Conditional Formatting to component instance
      component.measureInCFSeries = measureInCFSeries;
      component.categoryInCFSeries = categoryInCFSeries;
      if (measureDimensionIndex >= nDimensionInColumns) {
        component.columnInCFSeries = columnInCFSeries;
      }

      component.measureInCFSeries = measureInCFSeries;
      // code for changing the measure name when measure name is changed by the user
      if (component.chartmeasurechange() !== undefined && component.chartmeasurechange() !== '' && component.chartmeasurechange() !== null) {
        var xaxisname = JSON.parse(component.chartmeasurechange());
        for (var i = 0; i < seriesData.length; i++) {
          /*if(xaxisname[component.conditionalFormattingData[i].uniquekey] != '') {
                      seriesData[i].name = xaxisname[component.conditionalFormattingData[i].uniquekey];
                    }*/
          //modified by vinoth
          if (xaxisname[seriesData[i].uniquekey] !== '') {
            seriesData[i].name = xaxisname[seriesData[i]['uniquekey']];
          }
        }
      }

      component.conditionalFormattingData = seriesData;
      that.fullseriesData = seriesData;
    };
  }

  this.defaultvalueseries = function() {
    return that.defaultvalueseriesdata;
  };

  this.createErrorbar = function() {
    return that.fullseriesData;
  };

  if (validateImplementMethod(args, 'createSeriesData')) {
    that.createSeriesData = function(component, type, defaultValue/*, cfStatus*/) {
      var self = this;
      var includeResult = component.showtotaldatabase();
      /*
             * This block of code will map data to Dual Axes & Multiple Axes.
             */
      /*
             * Working: 1.Check Precondition & Proceed only if satisfy. 2. Generate
             * resultIndex[], feasibleColumn[], feasibleRow[] based on includeResult
             * Flag. 3. Check if DataUtilty is enabled. If yes go to 4.1 else 5.1.
             * 4.1. Check for dimension selected in data utility. If no selection
             * return. 4.2. Generate fixedColumnDimensions[] & fixedRowDimensions[] only for the
             * dimension selected in data utility. 4.3. Create columnMergeTuple[][] &
             * rowMergeTuple[][], a 2D array where subArray elements represent the
             * rows or columns to be aggregated into one row or column respectively.
             * Aggregate as information available in columnMergeTuple &
             * rowMergeTuple, generate series name, assign stack index and yAxis
             * index, and assign to seriesData. 5.1. Generate series name, assign
             * stack index, yaxis index and map data. 6. If the chart is Multiple
             * Axes, create yAxis object based on the yAxisLimit and add to
             * defaultValue
             */
      /*
             * Precondition to satisfy - a. Is Datasource availble. b. Is Measure
             * available. c. Is column available. d. Is feasible column available.
             */
      /*
             * Initiate variables - resultIndex[Length - No. of dimension, -1 if no
             * RESULT member else member index], feasibleColumn[Contains index of
             * column feasible], feasibleRow[Contains index of row feasible],
             * fixedColumnDimensions[Length - No. of dimension, -1 if dimension belong to row
             * else dimension index], fixedRowDimensions[Length - No. of dimension, -1 if
             * dimension belong to columns else dimension index]
             */
      /* Return null, if no data */
      if (isObjectEmpty(component.dataRes)) {
        return null;
      }
      /* Infer metaData of each selected point in runtime */
      component.categoriesIndex = [];
      /* Refers to each series chart type */
      var innerChartType = component.dataseriestype().split(',');
      var nInnerChartType = innerChartType.length;
      /* Refers to each series mapping to n-th yAxis */
      var seriesYaxis = [];
      if (!(component.dataseriesyaxis() === undefined || component.dataseriesyaxis() === '')) {
        seriesYaxis = component.dataseriesyaxis().split(',');
      }
      /* Retains innerChartType & yAxis mapping of each series during filtering in runtime */
      var measureTypePair;
      if (!(component.measuretypepair() === undefined || component.measuretypepair() === '')) {
        measureTypePair = JSON.parse(component.measuretypepair());
      }

      var DMchange = false;
      var dimName = component.getDimensionAndMeasure();
      if (component.oldDimname === undefined || component.oldDimname === '') {
        component.oldDimname = dimName.toString();
      }

      if (component.oldDimname !== dimName.toString()) {
        DMchange = true;
        if (sap.zen.designmode) {
          component.oldDimname = dimName.toString();
        }
      }
      /* Rules for conditional formatting */
      // @Conditional Formatting changes as per Amit's Requirements
      var measureInOriginalSeries = {};
      // After creation assign its data to 'component.measureInOriginalSeries'
      var categoryInOriginalSeries = {};
      // After creation assign its data to 'component.categoryInOriginalSeries'
      var columnInOriginalSeries = {};
      // After creation assign its data to 'component.columnInOriginalSeries'
      var selectedRowdimIndexes = [];
      // After creation assign its data to 'component.selectedRowdimIndexes'
      var selectedColumndimIndexes = [];
      // After creation assign its data to 'component.selectedColumndimIndexes'
      var ruleInfo = '';
      if (component.ruletable() !== undefined && component.ruletable() !== '') {
        that.updateRuleInfo(component, 'bar');
        ruleInfo = JSON.parse(component.ruletable());
      }
      /* Data variables for defaultValue */
      var seriesData = [];
      var seriesCategory = [];
      var xaxisCategoryKey = [];
      var measureCategory = [];
      /* Reference to dataSource JSON */
      var tupleMatrixInJSON = component.dataRes.tuples;
      var dataArrayInJSON = component.dataRes.data;
      var dimensionsInJSON = component.dataRes.dimensions;
      var columnsTupleInJSON = component.dataRes['axis_columns'];
      var rowsTupleInJSON = component.dataRes['axis_rows'];
      var nDimensions = component.dataRes.dimensions.length;
      var nDimensionInColumns = columnsTupleInJSON[0].indexOf(-1) == -1 ? columnsTupleInJSON[0].length : columnsTupleInJSON[0].indexOf(-1);
      var nColumns = component.dataRes.columnCount;
      var nRows = component.dataRes.rowCount;
      /* Flag variables */
      // If the result from data source is to include
      var dataUtilityEnabled = false;//component.datautility();
      var aggregateType = component.chartaggregation() || 'sum';
      var measureDimensionIndex = -1;
      // Index of measure dimension in dimension array of JSON
      /* working variables */
      var resultIndex = [];
      // Store RESULT member index, if any, for each dimension
      var fixedColumnDimensions = [];
      // dimension in columns, selected in data utility are marked 1 & others are -1
      var nFixedColumnDimensions = 0;
      var fixedRowDimensions = [];
      // dimension in rows, selected in data utility are marked 1 & others are -1
      var nFixedRowDimensions = 0;
      var feasibleColumn = [];
      // Stores feasible columns
      var feasibleRow = [];
      // Stores feasible rows
      var selectedColumn = [];
      // Stores selected columns
      var yAxisLimit = 0;
      // No. of Y-Axis to be generated
      /* conditional formatting comparator for 'string' operator */
      /*var comparator = {

                  // * a comparison value
                  // * b reference value
                   //* c reference value

        '>': function(a, b) {
          return _.parseInt(a) > _.parseInt(b);
        },

        '>=': function(a, b) {
          return _.parseInt(a) >= _.parseInt(b);
        },

        '<': function(a, b) {
          return _.parseInt(a) < _.parseInt(b);
        },

        '<=': function(a, b) {
          return _.parseInt(a) <= _.parseInt(b);
        },

        '==': function(a, b) {
          return _.parseInt(a) == _.parseInt(b);
        },

        '><': function(a, b, c) {
          return _.parseInt(a) > _.parseInt(b) ? _.parseInt(a) < _.parseInt(c) : false;
        }
      };*/
      /*
             * Initialisation Loop
             * -------------------
             *
             * Get measureDimensionIndex from the datasource; +[Identify index
             * of member with type RESULT in dimension]; +[Initialize array
             * value of fixedColumnDimensions & fixedRowDimensions to -1];
             * Note: fixedColumnDimensions & fixedRowDimensions will contains dimensions
             * that will be part of output series
             */
      for (var i = 0, measureDimensionIndexFlag = false; i < nDimensions; ++i) {
        /*
                 * Get index of measure dimension in measureDimensionIndex
                 */
        if (!measureDimensionIndexFlag) {
          if (dimensionsInJSON[i].containsMeasures !== undefined) {
            measureDimensionIndex = i;
            measureDimensionIndexFlag = true;
          }
        }
        /*
                 * Initialize fixedColumnDimensions & fixedRowDimensions Array to default value -1.
                 * -1 states not to include. Initially set all dimensions not to include. Later based
                 * on data utility selection include selected dimensions
                 */
        fixedColumnDimensions[i] = -1;
        fixedRowDimensions[i] = -1;
        resultIndex[i] = -1;
        /*
                 * Get the Result Member of each dimension and update the
                 * resultIndex Array with the index position of member with type 'RESULT' in the
                 * order Dimensions appear in the JSON
                 */
        for (var j = 0, l = dimensionsInJSON[i].members.length; j < l; ++j) {
          if (dimensionsInJSON[i].members[j].type == 'RESULT') {
            resultIndex[i] = j;
            break;
          }
        }
      }
      /**
             * Validation
             */
      /*
             * Return immediately if no Measure Dimension exists in the data source
             */
      if (measureDimensionIndex == -1) {
        // this.createSmokeScreen(component.$(),"No Measure Dimension. Place Measure Dimension in datasorce");
        if (window.console) {
          console.log('No Measure Dimension Exists in the Data Source - Returning');  // jshint ignore:line
        }

        return -1;
      }
      /*
             * Return immediately if no Columns exists in the selection
             */
      if (nColumns === 0) {
        // this.createSmokeScreen(component.$(),"No Columns Exists. Place Dimension in Column");
        if (window.console) {
          console.log('No Columns in DataSource - Returning');  // jshint ignore:line
        }

        return -2;
      } 
      /*
             * Create two arrays [feasibleColumn, feasibleRow] each having list of
             * columns and rows index that will be included in the output. If
             * includeResult is true, include all the rows and columns in the arrays
             * else only push those rows & columns which are not the Result Row or
             * Column by referencing resultIndex array
             */
      if (!includeResult) {
        /*
                 * includeResult is false, So include only non-result columns & rows
                 * in the feasibleColumn & feasibleRow
                 */
        /*
                 * Looping into all columns
                 */
        for (var m = 0; m < nColumns; ++m) {
          /*
                     * Loop thru all dimensions that make up the current column and
                     * check each member if it is result or not
                     */
          var matchedFlag = false;
          for (var n = 0; n < nDimensionInColumns; ++n) {
            /*
                         * If current member at index n of current dimension at
                         * index m matches the n-th index of resultIndex then its a
                         * result index and break the loop after setting the flag
                         * and add that index to feasible array only if the flag is
                         * reset
                         */
            if (tupleMatrixInJSON[m][n] == resultIndex[n]) {
              matchedFlag = true;
              break;
            }
          }

          if (!matchedFlag) {
            feasibleColumn.push(m);
          }
        }
        /*
                 * Looping into all columns
                 */
        for (var m = 0; m < nRows; ++m) {
          /*
                     * Loop thru all dimensions that make up the current rowand
                     * check each member if it is result or not
                     */
          var matchedFlag = false;
          for (var n = nDimensionInColumns; n < nDimensions; ++n) {
            /*
                         * If current member at index n of current dimension at
                         * index m matches the n-th index of resultIndex then its a
                         * result index and break the loop after setting the flag
                         * and add that index to feasible array only if the flag is
                         * reset
                         */
            if (tupleMatrixInJSON[m * nColumns][n] == resultIndex[n]) {
              matchedFlag = true;
              break;
            }
          }

          if (!matchedFlag) {
            feasibleRow.push(m);
          }
        }
      } else {
        /*
                 * includeResult is true, So include all columns & rows in the
                 * feasibleColumn & feasibleRow
                 */
        for (var m = 0; m < nColumns; ++m) {
          feasibleColumn.push(m);
        }

        for (var m = 0; m < nRows; ++m) {
          feasibleRow.push(m);
        }
      }
      /*
             * Get rows & columns dimension from data utility selection
             */
      var utilityColumnDimension = [];
      var utilityRowDimensions = [];
      /*
             * NEW IMPLEMENTATION
             * If dataUtility is enabled get selected row and column dimension.
             * If disabled, include all dimensions from row and column in
             * utilityRowDimensions & utilityColumnDimensions
             */
      if (dataUtilityEnabled) {
        var selectionList = component.bardatacustomization();
        /*
                 * Return if no selection in data utility
                 */
        if (selectionList === '' || selectionList == ';') {
          // this.createSmokeScreen(component.$(),"Please select Dimension in Data Utility");
          if (window.console) {
            console.log('No Selection of Dimension in Data Utility - returning');  // jshint ignore:line
          }

          return -3;
        }

        selectionList = JSON.parse(selectionList);
        // parse string to JSON
        // Update utilityColumnDimension with column selection
        for (var i = 0, iLimit = selectionList.COLUMNS.length; i < iLimit; ++i) {
          utilityColumnDimension.push(selectionList.COLUMNS[i].index);
        }
        // Update utilityRowDimension with row selection
        for (var i = 0, iLimit = selectionList.ROWS.length; i < iLimit; ++i) {
          utilityRowDimensions.push(selectionList.ROWS[i].index);
        }
        // Update the value to 1 stating included in fixedColumnDimensions
        for (var i = 0, iLimit = utilityColumnDimension.length; i < iLimit; ++i) {
          fixedColumnDimensions[utilityColumnDimension[i]] = 1;
          selectedColumndimIndexes.push(utilityColumnDimension[i]);
          ++nFixedColumnDimensions;
        }
        // Update the value to 1 stating included in fixedRowDimensions
        for (var i = 0, iLimit = utilityRowDimensions.length; i < iLimit; ++i) {
          fixedRowDimensions[utilityRowDimensions[i]] = 1;
          selectedRowdimIndexes.push(utilityRowDimensions[i]);
          ++nFixedRowDimensions;
        }

        if (measureDimensionIndex < nDimensionInColumns) {
          fixedColumnDimensions[measureDimensionIndex] = 1;
          selectedColumndimIndexes.push(measureDimensionIndex);
          ++nFixedColumnDimensions;
        } else {
          fixedRowDimensions[measureDimensionIndex] = 1;
          selectedRowdimIndexes.push(measureDimensionIndex);
          ++nFixedRowDimensions;
        }
      } else {
        for (var i = 0, iLimit = nDimensionInColumns; i < iLimit; ++i) {
          fixedColumnDimensions[i] = 1;
          selectedColumndimIndexes.push(i);
          ++nFixedColumnDimensions;
        }

        for (var i = nDimensionInColumns, iLimit = nDimensions; i < iLimit; ++i) {
          fixedRowDimensions[i] = 1;
          selectedRowdimIndexes.push(i);
          ++nFixedRowDimensions;
        }
      }
      /*
             * Create columnMergeTuples array that will have the subArray which
             * should be merged into one column. LOGIC:[Feasible Column Tuples
             * having same values at index that are set in fixedColumnDimensions
             * needs to be grouped]
             */
      var columnMergeTuples = [];
      // This will contain column to be merged
      var visitedColumns = [];
      var columnMergeCount = 0;
      for (var i = 0; i < feasibleColumn.length; ++i) {
        // Get next column if this is already column is already checked
        if (visitedColumns[i] !== undefined) {
          continue;
        }
        // Set current column value to looping index column
        var currentColumn = columnsTupleInJSON[feasibleColumn[i]];
        // Loop through all the remaining columns for checking
        for (var j = i; j < feasibleColumn.length; ++j) {
          // Get next column if this is already column is already checked
          if (visitedColumns[j] !== undefined) {
            continue;
          }
          // Set comparing column value to looping index column which needs to tested against current column
          var comparingColumn = columnsTupleInJSON[feasibleColumn[j]];
          /*
                     * Match currentColumn values with comparingColumn values at
                     * index fixedColumnDimensions to test if they have
                     * identical values. If matched, add comparingColumn index to columnMergeTuples
                     */
          var matched = true;
          for (var k = 0; k < nDimensionInColumns; ++k) {
            if (fixedColumnDimensions[k] == 1) {
              if (currentColumn[k] != comparingColumn[k]) {
                matched = false;
                break;
              }
            }
          }

          if (matched) {
            // If new column aggregating group is undefined, define it
            if (columnMergeTuples[columnMergeCount] === undefined) {
              columnMergeTuples[columnMergeCount] = [];
            }
            // Add the column to this group
            columnMergeTuples[columnMergeCount].push(feasibleColumn[j]);
            // Set this column as visited to avoid checking later for other column aggregating groups
            visitedColumns[j] = true;
          }
        }
        // Incrementing column aggregating group index to next value
        ++columnMergeCount;
      }
      /*
             * Set yAxisLimit variable to limit the number yAxis allowed in
             * chart. For Dual Axis Maximum yAxis allowed is 2
             */
      if (type != 'dualAxes') {
        yAxisLimit = columnMergeTuples.length - 1;
      } else {
        // Magic Number 2 is for Max yAxis for Dual Axis
        if (columnMergeTuples.length > 2) {
          yAxisLimit = 1;
        } else {
          yAxisLimit = columnMergeTuples.length - 1;
        }
      }
      /*
             * NEW IMPLEMENTATION
             * Populate selectedColumn with columnMergeTuple
             */
      for (var i = 0, iLimit = columnMergeTuples.length; i < iLimit; ++i) {
        selectedColumn.push(columnMergeTuples[i][0]);
      }
      /*
             * For keeping track of stack index allocated in the chart
             */
      var stacking = {
        column: [],
        area: [],
        areaspline: []
      };
      /*
             * Create Series object for master JSON Object & append to
             * defaultValue in the end of all manipulation. Loop thru
             * selectedColumns to create list of series object for each series
             * selected
             */
      for (var i = 0, yIndex = 0, iLimit = columnMergeTuples.length; i < iLimit; ++i) {
        /*
                 * Create Name for the series to be called in legends
                 */
        var tempName = '';
        var tempNameAsKey = '';
        var tempKeyvalue = '';
        var tempKey = '';
        var tempConditionalFormattingKey = '';
        var tempMeasureKey = '';
        var measureDimString = '';
        // Current column
        var tempColumn = columnsTupleInJSON[selectedColumn[i]];
        for (var k = 0; k < tempColumn.length; ++k) {
          if (fixedColumnDimensions[k] != -1) {
            if (k === measureDimensionIndex) {
              tempMeasureKey = dimensionsInJSON[k].members[tempColumn[k]].key;
              measureDimString = k + ':' + tempColumn[k];
            }
 
            if (tempName !== '' && tempName !== ' ') {
              tempName = tempName + '- ' + dimensionsInJSON[k].members[tempColumn[k]].text;//VBX141-7967 || dimensionsInJSON[k].members[tempColumn[k]].key;
              tempKey = tempKey + '-' + k + ':' + tempColumn[k];
              tempNameAsKey = tempNameAsKey + '/ ' + dimensionsInJSON[k].members[tempColumn[k]].key || dimensionsInJSON[k].members[tempColumn[k]].text;
              tempConditionalFormattingKey = tempConditionalFormattingKey + '-' + dimensionsInJSON[k].members[tempColumn[k]].key || dimensionsInJSON[k].members[tempColumn[k]].text;
            } else {
              tempName = dimensionsInJSON[k].members[tempColumn[k]].text; //VBX141-7967 || dimensionsInJSON[k].members[tempColumn[k]].key;
              tempKey = k + ':' + tempColumn[k];
              tempNameAsKey = dimensionsInJSON[k].members[tempColumn[k]].key || dimensionsInJSON[k].members[tempColumn[k]].text;
              tempKeyvalue = dimensionsInJSON[k].members[tempColumn[k]].key;
              tempConditionalFormattingKey = dimensionsInJSON[k].members[tempColumn[k]].key || dimensionsInJSON[k].members[tempColumn[k]].text;
            }
          }
        }
        // For Conditional Formatting
        if (tempMeasureKey !== '') {
          if (nFixedColumnDimensions == 1) {
            measureInOriginalSeries[tempMeasureKey] = i;
          } else {
            measureInOriginalSeries[tempMeasureKey] = measureInOriginalSeries[tempMeasureKey] || {};
            measureInOriginalSeries[tempMeasureKey][tempConditionalFormattingKey] = i;
          }
        }

        columnInOriginalSeries[tempConditionalFormattingKey] = i;
        var tempChartType = innerChartType[i % nInnerChartType].split('-');
        var tempStacking = '';
        // This contains info if the stacking is normal or percentage based
        var tempStack = i;
        var tempYaxis = seriesYaxis[i];
        // Retain old mapping in case filtering DS affects series order
        if (measureTypePair && measureTypePair[tempNameAsKey] !== undefined) {
          if (measureTypePair[tempNameAsKey].type !== undefined) {
            tempChartType = measureTypePair[tempNameAsKey].type.split('-');
          }

          if (measureTypePair[tempNameAsKey].yAxis !== undefined) {
            tempYaxis = measureTypePair[tempNameAsKey].yAxis;
          }
        }

        if (tempChartType.length !== 1) {
          // This is excuted if chart type have additional info of chart type
          tempStacking = tempChartType[2];
          // index 2 contains information regarding stacking
          var innerStacking = tempStacking === undefined ? 0 : 1;
          if (tempStacking === undefined || tempStacking === '') {
            tempStacking = 'normal';
          }

          if (stacking[tempChartType[0]][innerStacking] === undefined) {
            tempYaxis = +(tempYaxis === undefined ? yIndex > yAxisLimit ? yAxisLimit : yIndex++ : tempYaxis);
            stacking[tempChartType[0]][innerStacking] = [];
            stacking[tempChartType[0]][innerStacking][0] = i;
            stacking[tempChartType[0]][innerStacking][1] = tempYaxis;  //_.parseInt(seriesYaxis[i] === undefined? (i > yAxisLimit? yAxisLimit : i) : seriesYaxis[i]);
          }

          tempStack = stacking[tempChartType[0]][innerStacking][0];
          tempYaxis = stacking[tempChartType[0]][innerStacking][1];
        } else {
          tempYaxis = +(tempYaxis === undefined ? yIndex > yAxisLimit ? yAxisLimit : yIndex++ : tempYaxis);
        }
        /*
                 * Assign default values if required and declare other objects
                 */
        seriesData[i] = {
          data: [],
          key: tempKey,
          name: self.formatSeriesName(component, tempName, measureDimString, {}),
          nameAsKey: tempNameAsKey,
          keyvalue: tempKeyvalue,
          uniquekey: tempKeyvalue,
          //required for errorbar to identify unique selection of series
          yAxis: tempYaxis,
          type: tempChartType[0],
          stack: tempStack,
          stacking: tempStacking
        };
      }
      /*
             * Create rowMergeTuples array that will have the subArray which
             * should be merged into one row. LOGIC:[Same as columnMergeTuple]
             */
      var visitedRows = [];
      var rowMergeTuples = [];
      var rowMergeCount = 0;
      for (var i = 0; i < feasibleRow.length; ++i) {
        if (visitedRows[i] !== undefined) {
          continue;
        }

        var currentRow = rowsTupleInJSON[feasibleRow[i]];
        for (var j = i; j < feasibleRow.length; ++j) {
          if (visitedRows[j] !== undefined) {
            continue;
          }

          var comparingRow = rowsTupleInJSON[feasibleRow[j]];
          var matched = true;
          for (var k = nDimensionInColumns; k < nDimensions; ++k) {
            if (fixedRowDimensions[k] == 1) {
              if (currentRow[k] !== comparingRow[k]) {
                matched = false;
                break;
              }
            }
          }

          if (matched) {
            if (rowMergeTuples[rowMergeCount] === undefined) {
              rowMergeTuples[rowMergeCount] = [];
            }

            rowMergeTuples[rowMergeCount].push(feasibleRow[j]);
            visitedRows[j] = true;
          }
        }

        ++rowMergeCount;
      }
      /*
             * Now rowMergeTuple & columnMergeTuple Array contains the required
             * the merging rules. Loop through the list of sub-array of both
             * Array and merge sub-array as one row or column. Note for column
             * aggregation always it is sum
             */
      var aggregatedData = [];
      for (var i = 0; i < rowMergeTuples.length; ++i) {
        aggregatedData[i] = [];
        var tempName = '';
        var tempKey = '';
        var tempRow = rowsTupleInJSON[rowMergeTuples[i][0]];
        var dimensionMemberPair = '';
        var tempMeasureKey = '';
        var tempConditionalFormattingKey = '';
        for (var k = 0; k < tempRow.length; ++k) {
          if (fixedRowDimensions[k] != -1) {
            if (k == measureDimensionIndex) {
              tempMeasureKey = dimensionsInJSON[k].members[tempRow[k]].key;
            }

            if (tempName !== '' && tempName !== ' ') {
              if(dimensionsInJSON[k].members[tempRow[k]].text){
                  tempName = tempName + '-' + dimensionsInJSON[k].members[tempRow[k]].text; // VBX141-5817 || dimensionsInJSON[k].members[tempRow[k]].key;
              }
              tempKey = tempKey + '*' + dimensionsInJSON[k].members[tempRow[k]].key;
              dimensionMemberPair = dimensionMemberPair + '-' + k + ':' + tempRow[k];
              // jscs:disable
              // Code here will be ignored by JSCS.
              tempConditionalFormattingKey = tempConditionalFormattingKey + '\\' + dimensionsInJSON[k].members[tempRow[k]].key || dimensionsInJSON[k].members[tempRow[k]].text;  // jscs:enable
            } else {
              tempName = dimensionsInJSON[k].members[tempRow[k]].text; // VBX141-5817 || dimensionsInJSON[k].members[tempRow[k]].key;
              dimensionMemberPair = k + ':' + tempRow[k];
              tempKey = dimensionsInJSON[k].members[tempRow[k]].key;
              tempConditionalFormattingKey = dimensionsInJSON[k].members[tempRow[k]].key || dimensionsInJSON[k].members[tempRow[k]].text;
            }
          }
        }
        // For Conditional Formatting
        if (tempMeasureKey !== '') {
          if (nFixedRowDimensions == 1) {
            measureInOriginalSeries[tempMeasureKey] = i;
          } else {
            measureInOriginalSeries[tempMeasureKey] = measureInOriginalSeries[tempMeasureKey] || {};
            measureInOriginalSeries[tempMeasureKey][tempConditionalFormattingKey] = i;
          }
        }

        categoryInOriginalSeries[tempConditionalFormattingKey] = i;
        component.categoriesIndex.push(dimensionMemberPair);
        seriesCategory.push(tempName);
        xaxisCategoryKey.push(tempKey);
        measureCategory.push(tempMeasureKey);

        var workingArray = [];
    var partialAggregation = null;
        for (var k = 0; k < selectedColumn.length; ++k) {
          workingArray[k] = [];
          for (var j = 0; j < rowMergeTuples[i].length; ++j) {
            var mergeColumns = columnMergeTuples[k];
            var mergeColumnData = [];
            for (var p = 0, pLimit = mergeColumns.length; p < pLimit; ++p) {
              var datavalue = dataArrayInJSON[+(rowMergeTuples[i][j] * nColumns) +mergeColumns[p]]; //jshint ignore:line
               if(datavalue === null){
                continue;
              } 
              mergeColumnData.push(datavalue); 
            }

           if(mergeColumnData.length === 0){
            continue;
           }           
            partialAggregation = this.aggregateArray(mergeColumnData, 'sum');
            workingArray[k].push(partialAggregation);
          }
        }

        for (var k = 0; k < selectedColumn.length; ++k) {
          if (workingArray[k] === '' || workingArray[k].length === 0){
        aggregatedData[i][k] = null;
      }
          else{
      aggregatedData[i][k] = this.aggregateArray(workingArray[k], aggregateType);
      }

        }

        for (var k = 0; k < selectedColumn.length; ++k) {
          seriesData[k].data.push({y: aggregatedData[i][k]});
        }
      }
      // Assign required properties for Conditional Formatting to component instance
      component.measureInOriginalSeries = measureInOriginalSeries;
      component.categoryInOriginalSeries = categoryInOriginalSeries;
      component.selectedRowdimIndexes = selectedRowdimIndexes;
      component.selectedColumndimIndexes = selectedColumndimIndexes;
      if (measureDimensionIndex >= nDimensionInColumns) {
        component.columnInOriginalSeries = columnInOriginalSeries;
      }
      /*
             * Add yAxis dynamically for the n number of selectedColumns in yAxis
             * for multipleAxes
             */
      if (type == 'multipleAxes') {
        // Added flag for CHART CONTAINER implementation
        if (component.hasOwnProperty('flag')) {
          defaultValue.yAxis = [];
          for (var i = 0, iLimit = seriesData.length; i < iLimit; ++i) {
            defaultValue.yAxis[i] = {};  /*     if(i == 0){
                          defaultValue.yAxis[i]={};
                        }

                        else{
                          defaultValue.yAxis[i]={};
                          defaultValue.yAxis[i].opposite=true;
                        }*/
          }
        } else {
          var newPropertyValues = {
            yaxisceiling: component.multiyaxisceiling().split(',#'),
            yaxisfloor: component.multiyaxisfloor().split(',#'),
            yaxismax: component.multiyaxismax().split(',#'),
            yaxismin: component.multiyaxismin().split(',#'),
            yaxistitletext: component.yaxistitletext().split(',#'),
            yaxistickinterval: component.yaxistickinterval().split(',#'),
            yaxisminortickinterval: component.yaxisminortickinterval().split(',#'),
            //            yaxisplotbandfrom : component.yaxisplotbandfrom().split(',#'),
            //            yaxisplotbandto : component.yaxisplotbandto().split(',#'),
            //            yaxisplotbandlabeltext : component.yaxisplotbandlabeltext().split(',#'),
            //            yaxisplotlinevalue : component.yaxisplotlinevalue().split(',#'),
            //            yaxisplotlinelabeltext : component.yaxisplotlinelabeltext().split(',#'),
            yaxislabelprefix: component.yaxislabelprefix().split(',#'),
            yaxislabelsuffix: component.yaxislabelsuffix().split(',#'),
            tooltipvalueprefix: component.tooltipvalueprefix().split(',#'),
            tooltipvaluesuffix: component.tooltipvaluesuffix().split(',#')
          };
          var tempYaxis = [];
          var maxYaxis = 0;
          for (var i = 0, iLimit = seriesData.length; i < iLimit; ++i) {
            if (seriesData[i].yAxis > maxYaxis) {
              maxYaxis = seriesData[i].yAxis;
            }
          }

          for (var i = 0; i <= maxYaxis; ++i) {
            var tempAxis = JSON.parse(JSON.stringify(defaultValue.yAxis[0]));
            //To make sure labelprefix and suffix not getting undefined in not selected y-axis while scripting
            if (newPropertyValues.yaxislabelprefix[i] === 'undefined' || newPropertyValues.yaxislabelprefix[i] === 'undefined') {
              newPropertyValues.yaxislabelprefix[i] = '';
              newPropertyValues.yaxislabelprefix[i] = '';
            }

            tempAxis.ceiling = newPropertyValues.yaxisceiling[i] === '' || isNaN(newPropertyValues.yaxisceiling[i]) || newPropertyValues.yaxisceiling[i] === undefined ? null : +newPropertyValues.yaxisceiling[i];
            tempAxis.floor = newPropertyValues.yaxisfloor[i] === '' || isNaN(newPropertyValues.yaxisfloor[i])  || newPropertyValues.yaxisfloor[i] === undefined ? null : +newPropertyValues.yaxisfloor[i];
            tempAxis.min = newPropertyValues.yaxismin[i] === '' || isNaN(newPropertyValues.yaxismin[i]) || newPropertyValues.yaxismin[i] === undefined ? null : +newPropertyValues.yaxismin[i];
            tempAxis.max = newPropertyValues.yaxismax[i] === '' ||  isNaN(newPropertyValues.yaxismax[i]) || newPropertyValues.yaxismax[i]  === undefined ? null : +newPropertyValues.yaxismax[i];
            tempAxis.tickInterval = newPropertyValues.yaxistickinterval[i] === '' || newPropertyValues.yaxistickinterval[i] === undefined ? null : +newPropertyValues.yaxistickinterval[i];
            tempAxis.minorTickInterval = newPropertyValues.yaxisminortickinterval[i] === '' || newPropertyValues.yaxisminortickinterval[i] === undefined ? 'auto' : +newPropertyValues.yaxisminortickinterval[i];
            tempAxis.title.text = newPropertyValues.yaxistitletext[i] || null;
            tempAxis.labels.formatter =  function(){
                var prefix = component.yaxislabelprefix().split(",#");
                var suffix = component.yaxislabelsuffix().split(",#");
                var index = _.indexOf(this.chart.axes, this.axis);
                index -=1;
                if(!prefix[index]){
                  prefix = "";
                }else{
                  prefix = prefix[index];
                }
                 if(!suffix[index]){
                  suffix = "";
                }else{
                   suffix = suffix[index];
                }
                if(component.yaxislabelautoscale() === true){
                  var formattext = prefix + " " + vbiHighchartsDefaultValue.prototype.defaultLabelFormatter.call({
                      axis: this.axis,
                      value: this.value,
                      component: component
                  }) + " " + suffix;
                  return formattext;
                }else{
                  return prefix + " " + this.value + " " +  suffix;
                }

            };
            tempAxis.labels.format = null;
           // tempAxis.labels.format = component.yaxislabelautoscale() == true ? null : (newPropertyValues.yaxislabelprefix[i] || '') + component.yaxislabelformat() + (newPropertyValues.yaxislabelsuffix[i] || '');
            //          tempAxis.plotBands.label = newPropertyValues.yaxisplotbandlabeltext[i] || null;
            //          tempAxis.plotLines.label = newPropertyValues.yaxisplotlinelabeltext[i] || null;
            tempAxis.opposite = !component.yaxisopposite();
            delete tempAxis.offset;
            delete tempAxis.title.offset;
            delete tempAxis.title.margin;
            delete tempAxis.labels.align;
            delete tempAxis.labels.x;
            delete tempAxis.labels.y;
            if (i !== 0) {
              delete tempAxis.plotBands;
              delete tempAxis.plotLines;
            }

            tempYaxis.push(tempAxis);
          }

          tempYaxis[0].opposite = component.yaxisopposite();
          defaultValue.yAxis = tempYaxis;
        }
      }
      /**
             * due to issue with marker implementation - color assignment is done in series manipulation.
             * This mapping has to happen immediately after creating series and before applying conditonal formatting
             * Assign colors to series data respecting old assigned values for each series
             */
              function appendColor(m, d) {
                var ml = m.length;
                for (var i = m.length - 1; i >= 0; i--) {
                    var index = d.indexOf(m[i]);
                    if (index > -1) {
                        if (index == d.length - 1) {
                            m.push(d[0]);
                        } else {
                            m.push(d[index + 1]);
                        }
                        break;
                    }
                }
                if (ml == m.length) {
                    m.push(d[0]);
                }
                return m;
            }
      var colorArray = [], seriesColorArray = [], measureColorPair = {}, yAxisRef, tempColor;
      var defaultColor = '#B586DA,#FF6657,#F3A730,#946B66,#B0AD53,#7AC1DE,#F1C337,#566C6C,#A1CC54,#543F1E'.split(',');
      //var nDefaultColor = defaultColor.length;
      if (!(component.measurecolorpair() === undefined || component.measurecolorpair() === '')) {
        measureColorPair = JSON.parse(component.measurecolorpair());
      }

      if (component.dataseriescolor() === '' || component.dataseriescolor() === undefined) {
        seriesColorArray = defaultColor;
      } else {
        seriesColorArray = component.dataseriescolor().split(',');
      }

      for (var i = 0, j = 0, iLimit = seriesData.length; i < iLimit; ++i) {
        yAxisRef = defaultValue.yAxis[seriesData[i].yAxis];
        if (measureColorPair[seriesData[i].nameAsKey] !== undefined) {
          tempColor = measureColorPair[seriesData[i].nameAsKey].color;
        } else {
          if(seriesColorArray[i] === undefined){
            appendColor(seriesColorArray,defaultColor);
          }
          tempColor = seriesColorArray[i];
         // tempColor === undefined ? tempColor = defaultColor[j++ % nDefaultColor] : false; //jshint ignore:line
        }

        seriesData[i].color = tempColor;
        if (type != 'dualAxes') {
          if (yAxisRef.title) {
            yAxisRef.title.style.color = tempColor;
          }

          if (yAxisRef.labels) {
            yAxisRef.labels.style.color = tempColor;
            yAxisRef.labels.events= {
                click: function () {
                  component.seriesLibrary.axisClickEvent(this,component);
                }
              };
          }

          yAxisRef.lineColor = tempColor;
        }

        colorArray.push(tempColor);
      }

      component.dataseriescolor(colorArray.toString());
      // End - color assignment
      /**
       * Use below area to add common code for chart
       * ===========================================
       */
      // Adding Markers starts
      var measureMarkerPair;
      if (!(component.measuremarkerpair() === undefined || component.measuremarkerpair() === '')) {
        measureMarkerPair = JSON.parse(component.measuremarkerpair());
      }

      //var markerarray = component.addmarkers().split(',');
      for (var i = 0; i < seriesData.length; i++) {
        if (measureMarkerPair && measureMarkerPair[seriesData[i].nameAsKey] !== undefined && component.hasOwnProperty('enablemarker') && component.enablemarker()) {
          var colorvalue=component.dataseriescolor().split(',');
          if(measureColorPair && measureColorPair[seriesData[i].nameAsKey]){
             colorvalue = measureColorPair[seriesData[i].nameAsKey].color;
          }

          if(colorvalue.constructor === Array){
              colorvalue = colorvalue[i];
          }
          seriesData[i].marker = {
            enabled: measureMarkerPair[seriesData[i].nameAsKey].enabled,
            radius: parseFloat(measureMarkerPair[seriesData[i].nameAsKey].size),
            symbol: measureMarkerPair[seriesData[i].nameAsKey].symbol,
            fillColor: component.defaultmarkercolor() === false ? measureMarkerPair[seriesData[i].nameAsKey].color : colorvalue,
            states: {
              hover: {
                enabled: measureMarkerPair[seriesData[i].nameAsKey].hoverenable,
                radius: parseFloat(measureMarkerPair[seriesData[i].nameAsKey].size)
              }
            }
          };
          for (var j = 0; j < seriesData[i].data.length; j++) {
            //  alert(seriesData[i].type);
            if (seriesData[i].type.indexOf('line') > -1 || seriesData[i].type.indexOf('area') > -1) {
              seriesData[i].data[j].color = measureMarkerPair[seriesData[i].nameAsKey].color;
            }
          }
        } else {

          if (component.hasOwnProperty('enablemarker') && !component.enablemarker()) {
            seriesData[i].marker = {
            enabled: true,
            fillColor: seriesData[i].color
          };}

        }
      }
      // Adding Markers ends
      //Conditional formatting
      if (ruleInfo !== '') {
        try {
          that.createCFData(component, 'bar', component.fulljson());
          that.processConditionalFormatting(component, seriesData, ruleInfo, 'bar');
        } catch (e) {
        }
      }
      /*
             * Include a extra row for Total Result if showtotal is enabled.
             */
      if (component.showtotal()) {
        seriesCategory.push('Total Result');
        for (var i = 0, iLimit = seriesData.length; i < iLimit; ++i) {
          if (seriesData[i].type.indexOf('line') > -1 || seriesData[i].type.indexOf('area') > -1){
        seriesData[i].data[seriesData[i].data.length] = {y: this.aggregateArray(seriesData[i].data, aggregateType),color:  seriesData[i].marker.fillColor};
      }
          else{
        seriesData[i].data[seriesData[i].data.length] = {y: this.aggregateArray(seriesData[i].data, aggregateType)};
      }

        }
      }
      /*
             * Push the manipulated data into default value object referencing to
             * the highchart configuration object
             */
      component["getorginal_DS"] =  jQuery.extend(true, [], seriesData);
      component["getorginal_Category"] = jQuery.extend(true, [], seriesCategory);
      defaultValue.xAxis.categories = seriesCategory;
      defaultValue.series = seriesData;

      defaultValue.xAxis.measurecategories = measureCategory;
      /*   for(var i=0;i<dimensionsInJSON.length;i++)
          {
              if(dimensionsInJSON[i].containsMeasures)
              {

                for(var j=0;j<dimensionsInJSON[i].members.length;j++)
                {
                    defaultValue.xAxis.measurecategories.push(dimensionsInJSON[i].members[j].key);
                }

              }
          }*/

      //try{
      if (component.enablehierarchylabels() === true) {
        var selectedRowdimIndexes = [];
       /* if (component.datautility()) {
          selectedRowdimIndexes = selectedRowdimIndexes.concat(utilityRowDimensions);
        } else {*/
          for (var i = nDimensionInColumns; i < dimensionsInJSON.length; i++) {
            selectedRowdimIndexes.push(i);
          }
       /* }*/

        defaultValue.xAxis.categories = this.createHierarchyLabels(component.dataRes, selectedRowdimIndexes, component);
        /* Following code is required to manipulate the series according to the order of xAxis categories.
                     This case will occur when the row Dimension selected in Data Utility is not in the same order as in Initial View.
                  */
        var selRowInd = [];
        selRowInd = selRowInd.concat(selectedRowdimIndexes);
        selRowInd.sort(function(a, b) {
          return a - b;
        });

        if (selRowInd.join(',') != selectedRowdimIndexes.join(',')) {
          var newCategoriesOrder = [];
          $.each(defaultValue.xAxis.categories, function(i, v) {
            that.createFlatCategory(v, newCategoriesOrder, [v.name]);
          });

          var newcategoryDataRowAggr = [];
          $.extend(true, newcategoryDataRowAggr, seriesData);
          $.each(newCategoriesOrder, function(i, v) {
            var k = seriesCategory.indexOf(v);
            $.each(newcategoryDataRowAggr, function(ind, ele) {
              var val = ele.data[k];
              seriesData[ind].data[i] = val;
            });
          });
        }
      }
      //}catch(e){}
      defaultValue.xAxis.key = xaxisCategoryKey;
      that.defaultvalueseriesdata = seriesData;
      var seriesName = [
        [],
        []
      ];
      for (var i = 0, len = seriesData.length; i < len; ++i) {
        if (seriesData[i].name === undefined) {
          break;
        } else {
          seriesName[0].push(seriesData[i].name);
          seriesName[1].push(seriesData[i].nameAsKey);
        }
      }
      return seriesName;
    };
  }

  this.defaultvalueseries = function() {
    return that.defaultvalueseriesdata;
  };

  if (validateImplementMethod(args, 'globalgooglefont')) {
    /**
         * Apply google font to all parts of the chart
         * @param {object} component - Reference to component
         * @param {object} defaultValue - HighCharts values
         * @param {object} fonttest - Font Name
         * @return {object} defaultValue
         */
    that.globalgooglefont = function globalgooglefont(component, defaultValue, fonttest) {
      // alert('Accessed from google font own property');
      // globalgooglefont
      /*added below code to check whether the call is from "ChartContainer" if so then skip x and y axis styling
            as chartcontainer only support styling of title,subtitle,legend,tooltip,credits */
      var container = false;
      if (component[0].id.indexOf('CHARTCONTAINER') != -1) {
        container = true;
      }

      try {
        defaultValue.title.style.fontFamily = fonttest;
        defaultValue.subtitle.style.fontFamily = fonttest;
        if (container === false) {
          defaultValue.xAxis.labels.style.fontFamily = fonttest;
          defaultValue.xAxis.title.style.fontFamily = fonttest;
          defaultValue.xAxis.plotLines[0].label.style.fontFamily = fonttest;
          if (defaultValue.xAxis.plotBands[0] !== undefined) {
            defaultValue.xAxis.plotBands[0].label.style.fontFamily = fonttest;
          }
          for (var i = 0, iLimit = defaultValue.yAxis.length; i < iLimit; ++i) {
            defaultValue.yAxis[i].labels.style.fontFamily = fonttest;
            defaultValue.yAxis[i].title.style.fontFamily = fonttest;
             if (defaultValue.yAxis[i].plotLines !== undefined) {
              defaultValue.yAxis[i].plotLines[0].label.style.fontFamily = fonttest;
            }
           /* if (defaultValue.yAxis[i].plotBands) {
              defaultValue.yAxis[i].plotBands[0].label.style.fontFamily = fonttest;
            }*/
          }
        }
        if(container === true){
          if(defaultValue.xAxis){
                defaultValue.xAxis.labels.style={};
                defaultValue.xAxis.labels.style.fontFamily = fonttest;
           }
          if(defaultValue.yAxis){
              defaultValue.yAxis[0].title={};
              defaultValue.yAxis[0].title.style={};
              defaultValue.yAxis[0].title.style.fontFamily = fonttest;
            for (var i = 0, iLimit = defaultValue.yAxis.length; i < iLimit; ++i) {
              defaultValue.yAxis[i].labels={};
              defaultValue.yAxis[i].labels.style={};
              defaultValue.yAxis[i].labels.style.fontFamily = fonttest;
            }
          }
        }    
        defaultValue.tooltip.style.fontFamily = fonttest;
        defaultValue.legend.itemStyle.fontFamily = fonttest;
        defaultValue.legend.title.style.fontFamily = fonttest;
        defaultValue.legend.navigation.style.fontFamily = fonttest;
        defaultValue.credits.style.fontFamily = fonttest;
      } catch (e) {
      }

      return defaultValue;
    };
  }
}
